/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.core.xmms2;


import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observer;
import java.util.Random;

import org.x4x.core.ControlThread;
import org.x4x.core.KeepStateThread;
import org.x4x.core.ManagedThread;
import org.x4x.core.Management;
import org.x4x.core.Playlist;
import org.x4x.core.ReturnValue;
import org.x4x.core.Title;
import org.xmms2.xmms2bindings.*;


/**
 * Implementation of a xmms2 specific ControlThread. for Documentation on the various methods
 * have a look at core.ControlThread - noone likes to do things twice ;)
 */

public final class Xmms2ControlThread extends ManagedThread implements ControlThread {	
	private SWIGTYPE_p_xmmsc_connection_St myConnection;
	private SWIGTYPE_p_xmmsc_result_St result;
	private Xmms2KeepStateThread kst;
	
	protected Xmms2ControlThread(ReturnValue ret, Observer ob, Management mm, 
			Xmms2KeepStateThread kst, SWIGTYPE_p_xmmsc_connection_St connection){
		super(ret, ob, mm);
		this.kst = kst;
		myConnection = connection;
	}
	
	public ReturnValue play() {
		logger.debug("play");
		result = Xmmsclient.xmmsc_playback_start(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue stop() {
		logger.debug("stop");
		result = Xmmsclient.xmmsc_playback_stop(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue pause() {
		if (mm.getPlaybackStatus() == KeepStateThread.PLAYING)
			result = Xmmsclient.xmmsc_playback_pause(myConnection);
		else if (mm.getPlaybackStatus() == KeepStateThread.PAUSED)
			result = Xmmsclient.xmmsc_playback_start(myConnection);
		logger.debug("pause");
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue next() {
		if (kst.isRandomizing())
			return nextInPlaylistAbs(new Random().nextInt(mm.getCurrentPlaylist().getSize()));
		return nextInPlaylistRel(1);
	}
	
	public ReturnValue previous() {
		return nextInPlaylistRel(-1);
	}
	
	public ReturnValue seek(long ms) {
		result = Xmmsclient.xmmsc_playback_seek_ms(myConnection, (int)ms);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue nextInPlaylistAbs(int abs) {
		result = Xmmsclient.xmmsc_playlist_set_next(myConnection, abs);
		Xmmsclient.xmmsc_result_wait(result);
		if (mm.getPlaybackStatus() == KeepStateThread.PLAYING){
			Xmmsclient.xmmsc_result_unref(result);
			result = Xmmsclient.xmmsc_playback_tickle(myConnection);
			Xmmsclient.xmmsc_result_wait(result);
		}
		return ret;
	}
	
	public ReturnValue nextInPlaylistRel(int rel) {
		result = Xmmsclient.xmmsc_playlist_set_next_rel(myConnection, rel);
		Xmmsclient.xmmsc_result_wait(result);
		if (mm.getPlaybackStatus() == KeepStateThread.PLAYING){
			Xmmsclient.xmmsc_result_unref(result);
			result = Xmmsclient.xmmsc_playback_tickle(myConnection);
			Xmmsclient.xmmsc_result_wait(result);
		}
		return ret;
	}
	
	public ReturnValue playlistShuffle() {
		result = Xmmsclient.xmmsc_playlist_shuffle(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue playlistClear() {
		result = Xmmsclient.xmmsc_playlist_clear(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue playlistSort(String by) {
		/*result = Xmmsclient.xmmsc_playlist_sort(myConnection, by);
		Xmmsclient.xmmsc_result_wait(result);*/
		if (!by.equalsIgnoreCase("id"))
			sortPlaylist(mm.getCurrentPlaylist(), by);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylist(String path) {
		result = Xmmsclient.xmmsc_playlist_add(myConnection, path);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylistIndex(String path, int index) {
		result = Xmmsclient.xmmsc_playlist_insert(myConnection, index, path);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylist(long id) {
		result = Xmmsclient.xmmsc_playlist_add_id(myConnection, id);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addTitleToPlaylist(long id, int index) {
		result = Xmmsclient.xmmsc_playlist_insert_id(myConnection, index, id);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue addURLsToPlaylist(List urls) {
		if (urls != null){
			if (kst != null) kst.playlistBusy(true);
			for (Iterator it = urls.iterator(); it.hasNext();){
				String url = ""+it.next();
				ReturnValue rv = addTitleToPlaylist(url);
				if (rv.getType() == ReturnValue.ERROR_TYPE)
					return ret;
				Xmmsclient.xmmsc_result_unref(result);
			}
			result = null;
			if (kst != null) kst.playlistBusy(false);
		}
		return ret;
	}
	
	public ReturnValue addIDsToPlaylist(List ids) {
		if (ids != null){
			if (kst != null) kst.playlistBusy(true);
			for (Iterator it = ids.iterator(); it.hasNext();){
				try {
					Long id = (Long)it.next();
					ReturnValue rv = addTitleToPlaylist(id.longValue());
					if (rv.getType() == ReturnValue.ERROR_TYPE)
						return ret;
					Xmmsclient.xmmsc_result_unref(result);
				} catch (Exception e){
					logger.debug(e.getMessage(), e);
				}
			}
			result = null;
			if (kst != null) kst.playlistBusy(false);
		}
		return ret;
	}
	
	public ReturnValue addPathToPlaylist(String path) {
		List l = new ArrayList();
		recurseDir(new File(path), l);
		return addURLsToPlaylist(l);
	}
	
	/**
	 * Recurse over a given directory and fill the List
	 * 
	 * @param file
	 * @param l
	 */
	private void recurseDir(File file, List l) {
		if (file.exists() && file.isDirectory()){
			File list[] = file.listFiles(new FileFilter(){
				public boolean accept(File arg0) {
					if (arg0.isDirectory()) return true;
					String supported[] = mm.getFileTypes();
					for ( int i = 0; i < supported.length; i++)
						if (arg0.getName().endsWith(supported[i]))
							return true;
					return false;
				}
			});
			for ( int i = 0; i < list.length; i++){
				if (list[i].isDirectory())
					recurseDir(list[i], l);
				else l.add("file://" + list[i]);
			}
		}
	}
	
	public ReturnValue removeTitleFromPlaylist(long index) {
		result = Xmmsclient.xmmsc_playlist_remove(myConnection, (int)index);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue removeTitlesFromPlaylist(long indices[]) {
		for ( int i = 0; i < indices.length; i++)
			if (removeTitleFromPlaylist(indices[i]-i).getType() == ReturnValue.ERROR_TYPE)
				break;
		return ret;
	}
	
	public ReturnValue setSetting(String setting, String value) {
		result = Xmmsclient.xmmsc_configval_set(myConnection, setting, value);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue creaseVolume(int val) {
		logger.debug("crease volume");
		return setVolume(mm.getVolume() + val);
	}
	
	public ReturnValue setVolume(int val) {
		logger.debug("Set volume to " + val);
		result = Xmmsclient.xmmsc_playback_volume_set(myConnection, "left", val);
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_iserror(result) == 1){
			ret.setType(ReturnValue.ERROR_TYPE);
			ret.setValue(Xmmsclient.xmmsc_result_get_error(result));
		}
		if (result != null) Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playback_volume_set(myConnection, "right", val);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue setMixer(int mixer, int val) {
		if (mm.getConfigVal("equalizer.use_legacy").equals("1"))
			return setSetting("equalizer.legacy" + mixer, ""+(val/2.5));
		return setSetting("equalizer.gain" + ((mixer<10)?"0":"") + mixer, ""+(val/2.5));
	}
	
	public ReturnValue setPreamp(int val) {
		return setSetting("equalizer.preamp", ""+(val/2.5));
	}
	
	public ReturnValue setMixerEnabled(boolean flag){
		int i = 0;
		String conf = mm.getConfigVal("effect.order." + i);
		while (conf != null && !conf.equals("")){
			if (conf.equals("equalizer"))
				break;
			conf = mm.getConfigVal("effect.order." + ++i);
		}
		if (!mm.getConfigVal("equalizer.enabled").equals("1"))
			setSetting("equalizer.enabled", "1");
		return setSetting("effect.order." + i, (flag)?"equalizer":"");
	}
	
	public ReturnValue setRandomPlayback(boolean flag){
		result = Xmmsclient.xmmsc_configval_get(myConnection, "clients.x4x.randomizing");
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_iserror(result) == 1){
			Xmmsclient.xmmsc_result_unref(result);
			result = Xmmsclient.xmmsc_configval_register(myConnection, "x4x.randomizing", ""+!flag);
			Xmmsclient.xmmsc_result_wait(result);
		}
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_configval_set(myConnection, "clients.x4x.randomizing", ""+flag);
		Xmmsclient.xmmsc_result_wait(result);
		if (flag){
			setRepeatAll(false);
			setRepeatTrack(false);
		}
		return ret;
	}
	
	public ReturnValue setRepeatTrack(boolean flag){
		result = Xmmsclient.xmmsc_configval_set(myConnection, "playlist.repeat_one", ((flag)?"1":"0"));
		Xmmsclient.xmmsc_result_wait(result);
		if (flag){
			setRandomPlayback(false);
			setRepeatAll(false);
		}
		return ret;
	}
	
	public ReturnValue setRepeatAll(boolean flag){
		result = Xmmsclient.xmmsc_configval_set(myConnection, "playlist.repeat_all", ((flag)?"1":"0"));
		Xmmsclient.xmmsc_result_wait(result);
		if (flag) {
			setRandomPlayback(false);
			setRepeatTrack(false);
		}
		return ret;
	}
	
	public String spinDown(){
		String error = null;
		if (result != null){
			if (Xmmsclient.xmmsc_result_iserror(result) == 1)
				error = Xmmsclient.xmmsc_result_get_error(result);
			else
				Xmmsclient.xmmsc_result_unref(result);
		}
		synchronized(Xmms2ThreadFactory.lockControlConnection){
			Xmms2ThreadFactory.lockControlConnection.notify();
			Xmms2ThreadFactory.lockControlConnection = Boolean.FALSE;
		}
		return error;
	}
	
	protected boolean enterCritical(){
		synchronized(Xmms2ThreadFactory.lockControlConnection){
			while (Xmms2ThreadFactory.lockControlConnection.equals(Boolean.TRUE)) try {
				Xmms2ThreadFactory.lockControlConnection.wait();
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
				return false;
			}
			Xmms2ThreadFactory.lockControlConnection = Boolean.TRUE;
			return true;
		}
	}
	
	private void sortPlaylist(Playlist pl, String sorter){
		//kst.playlistBusy(true);
		List l = new ArrayList();
		for (Iterator plI = pl.Iterator(); plI.hasNext();)
			l.add(plI.next());
		sortiere(l, sorter);
		playlistClear();
		for (Iterator it = l.iterator(); it.hasNext(); ){
			Title t = (Title)it.next();
			result = Xmmsclient.xmmsc_playlist_add_id(myConnection, t.getID());
			Xmmsclient.xmmsc_result_wait(result);
			if (it.hasNext())
				Xmmsclient.xmmsc_result_unref(result);
		}
		//kst.playlistBusy(false);
	}
	
	private void sortiere(List arr, String cp){
		for ( int k = (arr.size()-1)/2; k >= 0; k--) {
			downheap(k,arr.size()-1, arr, cp);
		}
		for ( int k = arr.size()-1; k > 0; k--){
			swap(0,k,arr);
			downheap(0,k-1,arr, cp);
		}
	}
	
	private void downheap(int k, int n, List arr, String cp){
		int j;
		while ( 2*k <= n){
			j = 2*k;
			if ( j < (n) && ((Title)arr.get(j)).getAttribute(cp).
					compareToIgnoreCase(((Title)arr.get(j+1)).
					getAttribute(cp)) < 0) j++;
			if (((Title)arr.get(k)).getAttribute(cp).
					compareToIgnoreCase(((Title)arr.get(j)).getAttribute(cp)) < 0){
				swap(k,j,arr);
				k = j;
			}
			else return;
		}
	}
	
	private void swap(int one, int two, List arr){
		Object temp = arr.get(one);
		arr.set(one, arr.get(two));
		arr.set(two, temp);
	}
}
