/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.core.xmms2;

import java.net.URLDecoder;
import java.util.Observer;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import org.x4x.core.KeepStateThread;
import org.x4x.core.Management;
import org.x4x.core.Playlist;
import org.x4x.core.Title;
import org.x4x.core.UpdateUIType;
import org.xmms2.CallbacksListener;
import org.xmms2.JMain;
import org.xmms2.xmms2bindings.*;


/**
 * The Xmms2KeepStateThread handles the callbacks in java (implements org.xmms2.Callbacks). It is the xmms2-specific
 * version of our KeepStateThread.The KeepStateThread uses the java-Mainloop from org.xmms2.MainloopJava.
 * For an idea what the functions do look at core.KeepStateThread and org.xmms2.Callbacks, only some things
 * are explained here
 */

final class Xmms2KeepStateThread extends KeepStateThread implements CallbacksListener{
	private SWIGTYPE_p_xmmsc_connection_St myConnection;
	private JMain mlj;
	private Xmms2ThreadFactory tf;
	private Title workingTitle;
	private boolean randomizing, jumped = true;
	private int timeCnt = 0;
	private Random randomizer;
	
	/**
	 * The constructor sets the connection and calls the superconstructor. Then it starts itself as a thread
	 * 
	 * @param tf			the Xmms2ThreadFactory (for reconnecting)
	 * @param ob			thread's Observer which will be informed if a config-value changes
	 * @param mm			Management
	 * @param myConnection  connection to the xmms2 server
	 */
	protected Xmms2KeepStateThread(Xmms2ThreadFactory tf, Observer ob, Management mm,
			SWIGTYPE_p_xmmsc_connection_St myConnection){
		super(ob, mm);
		this.tf = tf;
		this.myConnection = myConnection;
		new Thread(this).start();
	}
	
	public void run(){
		init();
		while (true)
			try {Thread.sleep(30);} catch (InterruptedException e) {}
	}

	/**
	 * initiate needed startup data
	 *
	 */
	private void init() {
		randomizer = new Random();
		while (!tf.isConnected())try {Thread.sleep(300);} catch (InterruptedException e) {}
		updateAll();
		initMainloop();
	}

	/**
	 * Init Mainloop means to set all callbacks etc
	 *
	 */
	private void initMainloop() {
		mlj = new JMain(this, myConnection);
		mlj.setConfigvalChangedCallback(0);
		mlj.setMedialibEntryChangedCallback(0);
		mlj.setMedialibPlaylistLoadedCallback(0);
		mlj.setPlaybackCurrentIDCallback(0);
		mlj.setPlaybackStatusCallback(0);
		mlj.setPlaylistChangedCallback(0);
		mlj.setPlaylistCurrentPositionCallback(0);
		mlj.setPlaybackPlaytimeSignal(0);
		//mlj.setVisualizationDataSignal(0);
		// TODO only useful when adding a complete path
		//mlj.setMediareaderUnindexedSignal(0);
		mlj.setPlaybackVolumeChangedCallback(0);
		mlj.setMediainfoReaderStatusCallback(0);
		mlj.start();
	}
	
	public void restart(){
		init();
	}
	
	/**
	 * updateAll - update startup data using data from clientlib
	 * 
	 */
	private void updateAll(){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.xmmsc_playback_status(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYBACK_STATUS, 0);
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playback_current_id(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYBACK_ID, 0);
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playback_volume_get(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYBACK_VOLUME_CHANGED, 0);
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_playlist_list(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_PLAYLIST_CHANGED, 0);
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_configval_list(myConnection);
		Xmmsclient.xmmsc_result_notifier_set(result, 
				XmmsclientConstants.CALLBACK_CONFIGVAL_CHANGED, 0);
		Xmmsclient.xmmsc_result_unref(result);
	}
	
	protected void spinDown(){
		if (mlj != null){
			mlj.spinDown();
		}
	}
	
	protected void playlistBusy(boolean flag){
		if (flag){
			SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.xmmsc_broadcast_playlist_changed(myConnection);
			Xmmsclient.xmmsc_result_disconnect(result);
			Xmmsclient.xmmsc_result_unref(result);
		}
		else {
			SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.xmmsc_playlist_list(myConnection);
			Xmmsclient.xmmsc_result_notifier_set(result, XmmsclientConstants.CALLBACK_PLAYLIST_CHANGED, 0);
			Xmmsclient.xmmsc_result_unref(result);
			mlj.setPlaylistChangedCallback(0);
		}
	}
	protected void mlibBusy(boolean flag){}
	
	protected int getEQ(int x){
		try {
			if ((configs.get("equalizer.use_legacy").equals("1") && x > 9 ) ||
				x > Integer.parseInt(""+configs.get("equalizer.bands")) || x > 30) return -51;
		}catch (Exception e){
			e.printStackTrace();
			return -51;
		}
		String val = "";
		if (configs.get("equalizer.use_legacy").equals("1"))
			val = ""+configs.get("equalizer.legacy" + x);
		else val = ""+configs.get("equalizer.gain" + ((x<10)?"0":"") + x);
		if (!val.equals("") && !val.equals("null"))
			try {
				return (int)(Double.parseDouble(val)*2.5);
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
		return -51;
	}
	
	protected int getPreamp(){
		String val = ""+configs.get("equalizer.preamp");
		if (!val.equals("") && !val.equals("null"))
			try {
				return (int)(Double.parseDouble(val)*2.5);
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
		return -1;
	}
	
	protected String[] getFileTypes() {
		return new String[]{"mp3", "flac", "ogg", "wav"};
	}
	
	protected String[] getPlaylistTypes() {
		return new String[]{"pls", "m3u"};
	}
	
	protected String[] getAllSupportedTypes(){
		return new String[]{"mp3", "flac", "ogg", "wav", "pls", "m3u"};
	}
	
	protected boolean isRandomizing(){
		return randomizing;
	}
	
	protected boolean isRepeatOneEnabled(){
		return (configs.get("playlist.repeat_one") != null && configs.get("playlist.repeat_one").equals("1")?true:false);
	}
	
	protected boolean isRepeatAllEnabled(){
		return (configs.get("playlist.repeat_all") != null && configs.get("playlist.repeat_all").equals("1")?true:false);
	}
	
	protected boolean isMixerEnabled(){
		int i = 0;
		try {
			String conf = mm.getConfigVal("effect.order." + i);
			while (conf != null && !conf.equals("")){
				if (conf.equals("equalizer"))
					return configs.get("equalizer.enabled").equals("1");
				conf = mm.getConfigVal("effect.order." + ++i);
			}
		} catch (Exception e){}
		return false;
	}
	
	protected void lockTitle() {
		if (workingTitle != null){
			synchronized (workingTitle){
				while (workingTitle != null){
					try {
						workingTitle.wait();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				workingTitle = new Title();
			}
		}
		else workingTitle = new Title();
	}
	
	protected Title unlockTitle() {
		if (workingTitle != null){
			synchronized (workingTitle){
				Title t = workingTitle;
				workingTitle.notify();
				workingTitle = null;
				return t;
			}
		}
		return new Title();
    }
	
	public void callbackConfigvalChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		Xmmsclient.xmmsc_result_dict_foreach(result, XmmsclientConstants.CALLBACK_DICT_FOREACH_FUNCTION, 1);
	}
	
	public void callbackPlaybackStatus(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		long state[] = new long[1];
		Xmmsclient.xmmsc_result_get_uint(result, state);
		if (xmms_playback_status_t.swigToEnum((int)state[0]).equals(
				xmms_playback_status_t.XMMS_PLAYBACK_STATUS_STOP))
			setPlaybackStatus(IDLE);
		else if (xmms_playback_status_t.swigToEnum((int)state[0]).equals(
				xmms_playback_status_t.XMMS_PLAYBACK_STATUS_PLAY)) 
			setPlaybackStatus(PLAYING);
		else if (xmms_playback_status_t.swigToEnum((int)state[0]).equals(
				xmms_playback_status_t.XMMS_PLAYBACK_STATUS_PAUSE))
			setPlaybackStatus(PAUSED);
	}
	
	private  boolean isError(SWIGTYPE_p_xmmsc_result_St result) {
		if (Xmmsclient.xmmsc_result_iserror(result) == 1){
			logger.debug("got error");
			logger.debug(Xmmsclient.xmmsc_result_get_error(result));
			return true;
		}
		return false;
	}
	
	public void callbackPlaybackID(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		final long id[] = new long[1];
		Xmmsclient.xmmsc_result_get_uint(result, id);
		if (!mm.isTitleInCache(id[0])){
			setCurrentTitle(mm.getTitleForID(id[0]));
			new Timer().schedule(new TimerTask(){
				public void run(){
					setCurrentTitle(mm.getTitleForID(id[0]));
				}
			}, 4000);
		}
		else setCurrentTitle(mm.getTitleForID(id[0]));
		jumped = true;
	}
	
	public void callbackPlaybackVolumeChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		long right[] = new long[1];
		long left[] = new long[1];
		right[0] = 0;
		left[0] = 0;
		Xmmsclient.xmmsc_result_get_dict_entry_uint32(result, "right", right);
		Xmmsclient.xmmsc_result_get_dict_entry_uint32(result, "left", left);
		setVolume((int)(right[0]+left[0])/2);
	}
	
	public void callbackPlaylistChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		if (Xmmsclient.xmmsc_result_is_list(result) == 1){
			Playlist pl = new Playlist();
			Xmmsclient.xmmsc_result_list_first(result);
			long id[] = new long[1];
			while (Xmmsclient.xmmsc_result_list_valid(result) == 1){
				Xmmsclient.xmmsc_result_get_uint(result, id);
				Title t = new Title();
				t.setID(id[0]);
				pl.add(t);
				Xmmsclient.xmmsc_result_list_next(result);
			}
			if (!pl.getIDs().equals(mm.getCurrentPlaylist().getIDs()))
				setCurrentPlaylist(pl);
			if (pl.getSize() > 0){
				SWIGTYPE_p_xmmsc_result_St result2 = Xmmsclient.xmmsc_playlist_current_pos(myConnection);
				Xmmsclient.xmmsc_result_notifier_set(result2, XmmsclientConstants.CALLBACK_PLAYLIST_CURRENT_POSITION, 0);
				Xmmsclient.xmmsc_result_unref(result2);
			}
			else setCurrentPlaylistIndex(0);
		}
		else {
			SWIGTYPE_p_xmmsc_result_St result2 = Xmmsclient.xmmsc_playlist_list(myConnection);
			Xmmsclient.xmmsc_result_notifier_set(result2, 
					XmmsclientConstants.CALLBACK_PLAYLIST_CHANGED, 0);
			Xmmsclient.xmmsc_result_unref(result2);
		}
	}
	
	public void callbackMedialibEntryChanged(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		long id[] = new long[1];
		Xmmsclient.xmmsc_result_get_uint(result, id);
		mm.updateTitleCache((int)id[0]);
		setChanged();
		notifyObservers(new UpdateUIType(UpdateUIType.MLIB_ENTRY_CHANGED, new Long(id[0])));
		if (getPlayingTitle().getID() == id[0])
			setCurrentTitle(mm.getTitleForID(id[0]));
	}
	
	public  void callbackMedialibPlaylistLoaded(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		setChanged();
		notifyObservers(new UpdateUIType(UpdateUIType.PLAYLIST_LOADED, null));
	}
	
	public  void callbackPlaylistCurrentPosition(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		
		long[] pos = new long[1];
		Xmmsclient.xmmsc_result_get_uint(result, pos);
		setCurrentPlaylistIndex((int)pos[0]);
		jumped = true;
	}
	
	public  void callbackDisconnect(int error){
		tf.connect();
	}
	
	public void signalPlaybackPlaytime(long res, int user_data){
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		if (Xmmsclient.xmmsc_result_get_class(result).
				equals(xmmsc_result_type_t.XMMSC_RESULT_CLASS_SIGNAL)){
			long time[] = new long[1];
			Xmmsclient.xmmsc_result_get_uint(result, time);
			if (timeCnt%5 == 0)
				updatePlaytime((int)time[0]/1000.0);
			result = Xmmsclient.xmmsc_result_restart(result);
			Xmmsclient.xmmsc_result_unref(result);
			if (jumped && randomizing){
				long tracktime = getPlayingTitle().getLength()*1000;
				if (tracktime > 1000 && time[0] >= tracktime-150){
					jumped = false;
					SWIGTYPE_p_xmmsc_result_St randRes = 
						Xmmsclient.xmmsc_playlist_set_next(myConnection, 
								randomizer.nextInt(getCurrentPlaylist().getSize()));
					Xmmsclient.xmmsc_result_notifier_set(randRes, 
							XmmsclientConstants.SIGNAL_PLAYBACK_PLAYTIME, 0);
					Xmmsclient.xmmsc_result_unref(randRes);
				}
			}
		}
	}
	
	public void signalVisualizationData(long res, int user_data){
		/*final SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		
		result = Xmmsclient.xmmsc_result_restart(result);
		Xmmsclient.xmmsc_result_unref(result);
		*/
	}
	
	/**
	 * Callback is used to fill configlist on startup
	 */
	public void callbackDictForeachFunction(String key, int type, String value, int user_data) {
		if (user_data == 0){
			fillTitle(key, value);
		}
		else if (user_data == 1){
			if (key != null && !key.equals("")){
				if (!configs.containsKey(key))
					configs.put(key, value);
				else updateConfig(key, value);
				if (key.startsWith("equalizer.gain")){
					setChanged();
					notifyObservers(new UpdateUIType(UpdateUIType.VOLUME_OR_EQUALIZER_CHANGED, 
							new Integer(key.replaceAll("equalizer.gain", ""))));
				}
				if (key.startsWith("equalizer.legacy")){
					setChanged();
					notifyObservers(new UpdateUIType(UpdateUIType.VOLUME_OR_EQUALIZER_CHANGED, 
							new Integer(key.replaceAll("equalizer.legacy", ""))));
				}
				if (key.startsWith("equalizer.preamp")){
					setChanged();
					notifyObservers(new UpdateUIType(UpdateUIType.VOLUME_OR_EQUALIZER_CHANGED, 
							"p"));
				}
				if (key.equals("clients.x4x.randomizing")){
					try {
						if (value != null && value.equals("true"))
							randomizing = true;
						else randomizing = false;
					} catch (Throwable t){
						logger.debug(t.getMessage(), t);
					}
				}
				if (key.equals("clients.x4x.randomizing") ||
						key.equals("playlist.repeat_all") ||
						key.equals("playlist.repeat_one") ||
						key.equals("equalizer.enabled") ||
						(key.startsWith("effect.order.") && value.equals("equalizer"))){
					setChanged();
					notifyObservers(new UpdateUIType(UpdateUIType.PLAYBACK_SETTING_CHANGED, null));
				}
					
			}
		}
		else System.out.println("dict: " + key + "\t" + value);
	}
	
	public void callbackMediainfoReaderStatus(long arg0, int arg1) {
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(arg0);
		if (isError(result))
			return;
		int state[] = new int[1];
		Xmmsclient.xmmsc_result_get_int(result, state);
		
		if (xmms_mediainfo_reader_status_t.swigToEnum(state[0]).equals(
				xmms_mediainfo_reader_status_t.XMMS_MEDIAINFO_READER_STATUS_RUNNING))
			setBackgroundWorkerStatus(UpdateUIType.BG_WORKING);
		else setBackgroundWorkerStatus(UpdateUIType.BG_IDLE);
	}
	
	public void signalMediareaderUnindexed(long res, int arg1) {
		SWIGTYPE_p_xmmsc_result_St result = Xmmsclient.getResultFromPointer(res);
		if (isError(result))
			return;
		long todo[] = new long[1];
		Xmmsclient.xmmsc_result_get_uint(result, todo);
		
		setBackgroundWorkerStatus(UpdateUIType.BG_WORKING+((int)todo[0]));
		
		result = Xmmsclient.xmmsc_result_restart(result);
		Xmmsclient.xmmsc_result_unref(result);
	}
	
	public void callbackPropdictForeachFunction(String key, int type, String value, String source, int user_data) {
		if (user_data == 0){
			fillTitle(key, value);
		}
	}
	
	private void fillTitle(String key, String value){
		if (workingTitle != null && key != null && !key.equals("")) {
			try {
				if (key.equals("duration"))
					workingTitle.setLength(Integer.parseInt(value)/1000);
				else if (key.equals("id"))
					workingTitle.setID(Long.parseLong(value));
				else if (key.equals("url")){
					try {
						workingTitle.setAttribute(key, URLDecoder.decode(value, "UTF-8"));
					} catch (Exception e){
						workingTitle.setAttribute(key, value);
					}
				}
				else workingTitle.setAttribute(key, value);
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
		}
	}
	
	public void lockFunction(int v) {}
	public void unlockFunction(int v) {}
	public void userDefinedCallback1(long arg0, int arg1) {}
	public void callbackMedialibEntryAdded(long arg0, int arg1) {}
	public void userDefinedCallback2(long arg0, int arg1) {}
	public void userDefinedCallback3(long arg0, int arg1) {}
}
