/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.core.xmms2;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.Observer;
import java.util.Set;

import org.x4x.core.ManagedThread;
import org.x4x.core.Management;
import org.x4x.core.MedialibThread;
import org.x4x.core.PlaylistInformation;
import org.x4x.core.ReturnValue;
import org.x4x.core.Title;
import org.x4x.core.UpdateUIType;
import org.xmms2.xmms2bindings.*;

/**
 * To get an idea what the methods do have a look at core.ManagedThread and core.MedialibThread
 */
public final class Xmms2MedialibThread extends ManagedThread implements MedialibThread {
	private SWIGTYPE_p_xmmsc_connection_St myConnection;
	private SWIGTYPE_p_xmmsc_result_St result;
	private Xmms2KeepStateThread kst;
	
	protected Xmms2MedialibThread(ReturnValue ret, Observer obs, Management mm, Xmms2KeepStateThread kst, 
			SWIGTYPE_p_xmmsc_connection_St connection) {
		super(ret, obs, mm);
		this.kst = kst;
		myConnection = connection;
	}
	
	public ReturnValue mlibAddEntry(String url) {
		logger.debug("mlibAddEntry");
		result = Xmmsclient.xmmsc_medialib_add_entry(myConnection,url);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue mlibAddURLs(List urls) {
		if (urls != null){
			if (kst != null) kst.mlibBusy(true);
			for (Iterator it = urls.iterator(); it.hasNext();){
				String url = ""+it.next();
				ReturnValue rv = mlibAddEntry(url);
				if (rv.getType() == ReturnValue.ERROR_TYPE)
					return ret;
				Xmmsclient.xmmsc_result_unref(result);
			}
			result = null;
			if (kst != null) kst.mlibBusy(false);
		}
		return ret;
	}

	public ReturnValue mlibAddPath(String path) {
		result = Xmmsclient.xmmsc_medialib_path_import(myConnection, path);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	public ReturnValue mlibAddQueryToPlaylist(String sql){
		sql = "SELECT id FROM Media " + sql;
		logger.debug("SQL add to playlist: " + sql);
		result = Xmmsclient.xmmsc_medialib_add_to_playlist(myConnection, sql);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	/**
	 * Check for errors in the passed result
	 * @param res
	 * @return		if error return true, false otherwise
	 */
	private boolean isError(SWIGTYPE_p_xmmsc_result_St res){
		if (Xmmsclient.xmmsc_result_iserror(res) == 1){
			System.out.println(Xmmsclient.xmmsc_result_get_error(res));
			return true;
		}
		return false;
	}
	
	/**
	 * Build title with id from result
	 * @param res
	 * @param id
	 * @return		hopefully a completely filled Title
	 */
	private Title getTitleForID(SWIGTYPE_p_xmmsc_result_St res, long id){
		Xmmsclient.xmmsc_result_wait(res);
		kst.lockTitle();
		Xmmsclient.xmmsc_result_propdict_foreach(res, 
				XmmsclientConstants.CALLBACK_PROPDICT_FOREACH_FUNCTION, 0);
		Title t = kst.unlockTitle();
		t.setID(id);
		return t;
	}
	
	/**
	 * Get Playlistinformation for a given id
	 * 
	 * @param id
	 * @return		new Playlistinformation
	 */
	private PlaylistInformation getPlaylistInformationForID(long id){
		String query = "SELECT entry, count(entry) AS cnt FROM Playlistentries WHERE playlist_id = " + id;
		logger.debug(query);
		int items[] = new int[1];
		String name[] = new String[1];
		name[0] = "";
		
		result = Xmmsclient.xmmsc_medialib_select(myConnection, query);
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_is_list(result) == 1){
			Xmmsclient.xmmsc_result_list_first(result);
			Xmmsclient.xmmsc_result_get_dict_entry_int32(result,"cnt", items);
		}
		
		return new PlaylistInformation(id, name[0], items[0]);
	}

	public ReturnValue mlibGetPlaylists() {
		String myQuery = "SELECT name, id FROM Playlist ORDER BY id;";
		logger.debug(myQuery);
		result = Xmmsclient.xmmsc_medialib_select(myConnection, myQuery);
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_is_list(result) == 1){
			Xmmsclient.xmmsc_result_list_first(result);
			int id[] = new int[1];
			String name[] = new String[1];
			name[0] = "";
			List entries = new LinkedList();
			Map ids = new HashMap();
			while (Xmmsclient.xmmsc_result_list_valid(result) == 1){
				Xmmsclient.xmmsc_result_get_dict_entry_int32(result, "id", id);
				Xmmsclient.xmmsc_result_get_dict_entry_str(result,"name", name);
				Xmmsclient.xmmsc_result_list_next(result);
				Integer tmpint = new Integer(id[0]);
				if(!ids.containsKey(tmpint) && !name[0].startsWith("_"))
					ids.put(new Integer(id[0]),name[0]);
			}
			Set tmpIds = ids.keySet();
			for (Iterator i = tmpIds.iterator(); i.hasNext();){
				Object current = i.next();
				PlaylistInformation tmp = getPlaylistInformationForID(((Integer)current).intValue());
				if (tmp != null) {
					tmp.setName((String)ids.get(current));
					entries.add(tmp);
				}
			}
			ret.setType(ReturnValue.PLAYLISTINFORMATION_TYPE);
			ret.setValue(entries);
			return ret;
		}
		ret.setType(ReturnValue.ERROR_TYPE);
		return ret;
	}

	public ReturnValue mlibSelect(String query, boolean search) {
		if (search)
			query = "SELECT DISTINCT id FROM Media " + query;
		else query = "SELECT DISTINCT value FROM Media " + query;
		
		logger.debug(query);
		result = Xmmsclient.xmmsc_medialib_select(myConnection, query);
		Xmmsclient.xmmsc_result_wait(result);
		if (Xmmsclient.xmmsc_result_is_list(result) == 1){
			Xmmsclient.xmmsc_result_list_first(result);
			int id[] = new int[1];
			String value[] = new String[1];
			List entries = new LinkedList();
			Set queryResults = new HashSet();
			while (Xmmsclient.xmmsc_result_list_valid(result) == 1){
				if (search) {
					Xmmsclient.xmmsc_result_get_dict_entry_int32(result,"id", id);
					Integer tmpint = new Integer(id[0]);
					if(!queryResults.contains(tmpint))
						queryResults.add(new Integer(id[0]));
				}
				else {
					Xmmsclient.xmmsc_result_get_dict_entry_str(result,"value", value);
					if(!queryResults.contains(value[0]))
						queryResults.add(value[0]);
				}
				Xmmsclient.xmmsc_result_list_next(result);
			}
			if (search){
				for (Iterator i = queryResults.iterator(); i.hasNext();){
					long nid = ((Integer)i.next()).longValue();
					SWIGTYPE_p_xmmsc_result_St res = Xmmsclient.xmmsc_medialib_get_info(myConnection, nid);
					Title tmp = getTitleForID(res, nid);
					Xmmsclient.xmmsc_result_unref(res);
					if (tmp != null){
						entries.add(tmp);
						setChanged();
						ReturnValue rv = new ReturnValue(-1, System.currentTimeMillis());
						rv.setType(ReturnValue.TITLE_TYPE);
						rv.setValue(tmp);
						notifyObservers(rv);
					}
				}
			}
			else entries.addAll(queryResults);
			if (search){
				ret.setType(ReturnValue.MEDIALIB_SEARCH_TYPE);
			}
			else ret.setType(ReturnValue.MEDIALIB_TYPE);
			ret.setValue(entries);
			return ret;
		}
		ret.setType(ReturnValue.ERROR_TYPE);
		return ret;
	}

	public ReturnValue mlibLoadPlaylist(String name) {
		result = Xmmsclient.xmmsc_playlist_clear(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		return mlibAddPlaylistToCurrent(name);
	}
	
	public ReturnValue mlibLoadPlaylist(String url, String name) {
		result = Xmmsclient.xmmsc_medialib_playlist_import(myConnection, name, url);
		Xmmsclient.xmmsc_result_wait(result);
		// TODO workaround until xmms2 team gets a mlib_playlist_added broadcast
		setChanged();
		notifyObservers(new UpdateUIType(UpdateUIType.PLAYLIST_LOADED, null));
		return ret;
	}
	
	public ReturnValue mlibAddPlaylistToCurrent(String name) {
		result = Xmmsclient.xmmsc_medialib_playlist_load(myConnection, name);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;		
	}
	
	public ReturnValue mlibDeletePlaylist(String name) {
		result = Xmmsclient.xmmsc_medialib_playlist_remove(myConnection, name);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;	
	}

	public ReturnValue mlibRenamePlaylist(String nameold, String namenew) {
		this.mlibLoadPlaylist(nameold);
		
		result = Xmmsclient.xmmsc_medialib_playlist_save_current(myConnection, namenew);
		Xmmsclient.xmmsc_result_wait(result);
	
		return mlibDeletePlaylist(nameold);	
	}

	public ReturnValue mlibNewPlaylist(String name) {
		result = Xmmsclient.xmmsc_playlist_clear(myConnection);
		Xmmsclient.xmmsc_result_wait(result);
		result = Xmmsclient.xmmsc_medialib_playlist_save_current(myConnection, name);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;	
	}

	public ReturnValue mlibSaveCurrentPlaylist(String name) {
		result = Xmmsclient.xmmsc_medialib_playlist_save_current(myConnection, name);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;	
	}

	public ReturnValue mlibGetTitleForID(long id) {
		result = Xmmsclient.xmmsc_medialib_get_info(myConnection, id);
		ret.setType(ReturnValue.TITLE_TYPE);
		ret.setValue(getTitleForID(result, id));
		return ret;
	}

	public ReturnValue mlibUpdateTitle(Title t) {
		/*if (t.getAttribute("url").toLowerCase().endsWith(".mp3") || t.getAttribute("url").toLowerCase().endsWith(".ogg")){
			try {
				updateMP3(t);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TagException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			result = Xmmsclient.xmmsc_medialib_entry_property_set(myConnection, 
					t.getID(), "url", t.getAttribute("url"));
			Xmmsclient.xmmsc_result_wait(result);
			return mlibRefresh(t.getID());
		}
		Xmmsclient.xmmsc_result_unref(result);*/
		result = Xmmsclient.xmmsc_medialib_entry_property_set_str_with_source(myConnection, 
				t.getID(), Xmms2ThreadFactory.mySource, "artist", t.getAttribute("artist"));
		Xmmsclient.xmmsc_result_wait(result);
		if (isError(result)) return ret;
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_medialib_entry_property_set_str_with_source(myConnection, 
				t.getID(), Xmms2ThreadFactory.mySource, "album", t.getAttribute("album"));
		Xmmsclient.xmmsc_result_wait(result);
		if (isError(result)) return ret;
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_medialib_entry_property_set_str_with_source(myConnection, 
				t.getID(), Xmms2ThreadFactory.mySource, "title", t.getAttribute("title"));
		Xmmsclient.xmmsc_result_wait(result);
		if (isError(result)) return ret;
		Xmmsclient.xmmsc_result_unref(result);
		result = Xmmsclient.xmmsc_medialib_entry_property_set_str_with_source(myConnection, 
				t.getID(), Xmms2ThreadFactory.mySource, "comment", t.getAttribute("comment"));
		Xmmsclient.xmmsc_result_wait(result);
		mm.updateTitleCache(t.getID());
		return ret;
	}
	
	public ReturnValue mlibRefresh() {
		return mlibRefresh(0);
	}
	
	/**
	 * Refresh some specific medialib id
	 * @param id
	 * @return		empty ReturnValue
	 */
	private ReturnValue mlibRefresh(int id){
		result = Xmmsclient.xmmsc_medialib_rehash(myConnection, id);
		Xmmsclient.xmmsc_result_wait(result);
		return ret;
	}
	
	/*
	 * not used atm
	 * @param t
	 * @throws IOException
	 * @throws TagException
	 */
	/*private void updateMP3(Title t) throws IOException, TagException{
		// TODO doesn't work atm. For later: don't depend directly on it
		// but try to load the classvia class.findClass() instead of 
		// depending on it directly to avoid jvm errors if it's not
		// present
		System.out.println(t.getAttribute("url"));
		File f = null;
		try {
			f = new File(URLDecoder.decode(t.getAttribute("url"), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			logger.debug(e.getMessage(), e);
		} 
		if (f.exists()){
			logger.debug("gonna update " + f);
			MP3File mp3 = new MP3File(f);
			ID3v1 tag = mp3.getID3v1Tag();
			if (tag != null){
				System.out.println("updating " + mp3);
				tag.setAlbum(t.getAttribute("album"));
				tag.setArtist(t.getAttribute("artist"));
				tag.setComment(t.getAttribute("comment"));
				tag.setTitle(t.getAttribute("title"));
				mp3.setID3v1Tag(tag);
				mp3.save();
			}
		}
	}*/
	
	protected String spinDown(){
		String error = null;
		if (result != null){
			if (Xmmsclient.xmmsc_result_iserror(result) == 1)
				error = Xmmsclient.xmmsc_result_get_error(result);
			else
				Xmmsclient.xmmsc_result_unref(result);
		}
		synchronized (Xmms2ThreadFactory.lockMedialibConnection){
			Xmms2ThreadFactory.lockMedialibConnection.notify();
			Xmms2ThreadFactory.lockMedialibConnection = Boolean.FALSE;
		}
		return error;
	}
	
	protected boolean enterCritical(){
		synchronized(Xmms2ThreadFactory.lockMedialibConnection){
			while (Xmms2ThreadFactory.lockMedialibConnection.equals(Boolean.TRUE)){
				try {
					Xmms2ThreadFactory.lockMedialibConnection.wait();
				} catch (InterruptedException e) {
					logger.debug(e.getMessage(), e);
					return false;
				}
			}
			Xmms2ThreadFactory.lockMedialibConnection = Boolean.TRUE;
			return true;
		}
	}
}
