/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.util.ArrayList;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XCheckBox;
import org.Xwidgets.XGridData;
import org.Xwidgets.XGridLayout;
import org.Xwidgets.XMenu;
import org.Xwidgets.XPanel;
import org.Xwidgets.XSlider;
import org.apache.log4j.Logger;
import org.jdom.Element;
import org.x4x.core.Management;
import org.x4x.statics.StaticValues;

/**
 * EQPanel represents the UI-Part of the Equalizer. It provides a XPanel with n Equalizer sliders and labels.
 * It is, as every other ui-class singleton.
 */

class EQPanel {
	private static EQPanel instance = null;
	protected XPanel panel;
	protected XMenu menu;
	protected XGridLayout layout;
	protected XGridData layoutData, enbabledData;
	private Logger logger;
	private WidgetFactory wt;
	protected ArrayList sliders;
	protected int eqCount = 0;
	protected XCheckBox enabled;
	protected XSlider preamp;
	
	/**
	 * Constructor for EQPanel - usually called only once cause it's singleton
	 * 
	 * @param parent
	 * @param panel
	 * @param eqCount
	 */
	private EQPanel(XPanel parent, Element panel, int eqCount){
		try {
			logger = Logger.getLogger(getClass());
			wt = MainGUI.mainGui.myGUIFactory;
			this.eqCount = eqCount;
			setLayout();
			buildMe(parent, panel);
		}catch (Exception e){
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Set layoutdata for panels
	 *
	 */
	private void setLayout() {
		layout = wt.getGridLayout();
		layout.setHorizontalSpacing(0);
		layout.setColumnNums(eqCount+2);
		layout.makeColumnsEqualWidth(false);
		
		layoutData = wt.getGridData();
		layoutData.setExcessHorizontalSpace(true);
		layoutData.setExcessVerticalSpace(true);
		layoutData.setHorizontalAlignment(Styles.FILL);
		layoutData.setVerticalAlignment(Styles.FILL);
		
		enbabledData = wt.getGridData();
		enbabledData.setHorizontalAlignment(Styles.FILL);
		enbabledData.setHorizontalSpan(eqCount+2);
	}

	/**
	 * Create or return the existing EQPanel
	 * 
	 * @param parent		the EQPanel's parent
	 * @param panel		jdom Element describing the EQPanel
	 * @param eqCount		No of Equalizers (usually got from the Management)
	 * @return				a EQPanel object
	 */
	
	protected static EQPanel getEQPanel(XPanel parent, Element panel, int eqCount){
		if (instance == null)
			instance = new EQPanel(parent, panel, eqCount);
		return instance;
	}

	/**
	 * Called by Constructor to initialize the thing
	 * 
	 * @param parent
	 * @param p
	 */
	private void buildMe(XPanel parent, Element p) {
		try {
			sliders = new ArrayList();
			Management myManagement = Management.getInstance();
			
			panel = wt.getPanel(parent, Styles.SIMPLE);
			panel.setLayout(layout);
			panel.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			enabled = wt.getCheckBox(panel, Styles.LEFT);
			enabled.setChecked(myManagement.isMixerEnabled());
			enabled.setText(StaticValues.getLangString("enabled"));
			enabled.setLayoutData(enbabledData);
			enabled.addMouseListener(MouseListener.getInstance());
			
			preamp = wt.getSlider(panel, Styles.VERTICAL);
			preamp.addMouseListener(MouseListener.getInstance());
			preamp.setMax(100);
			preamp.setPosition((myManagement.getPreamp()-50)*(-1));
			preamp.setToolTipText("Preamp: "+myManagement.getPreamp());
			preamp.setLayoutData(layoutData);
			
			XPanel tmp = wt.getPanel(panel, Styles.SIMPLE);
			XGridData xgd = wt.getGridData();
			xgd.setWidthHint(20);
			xgd.setExcessHorizontalSpace(true);
			xgd.setExcessVerticalSpace(true);
			xgd.setHorizontalAlignment(Styles.FILL);
			xgd.setVerticalAlignment(Styles.FILL);
			tmp.setLayoutData(xgd);
			
			for (int i = 0; i < eqCount; i++){
				XSlider s = wt.getSlider(panel, Styles.VERTICAL);
				s.addMouseListener(MouseListener.getInstance());
				s.setMax(100);
				s.setPosition((myManagement.getEQ(i)-50)*(-1));
				s.setToolTipText(""+myManagement.getEQ(i));
				s.setLayoutData(layoutData);
				sliders.add(s);
			}
			
			if (MainGUI.mainGui.volume == null){
				MainGUI.mainGui.volume = wt.getSlider(parent, Styles.HORIZONTAL);
				MainGUI.mainGui.volume.setMax(100);
				MainGUI.mainGui.volume.setLayoutData(MainGUI.mainGui.layoutDataFillHoriz);
				MainGUI.mainGui.volume.addMouseListener(MouseListener.getInstance());
			}
			
			menu = wt.getDropDownMenu(panel);
			panel.setMenu(menu);
		} catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
	}
}
