/******************************************************************
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/


package org.x4x.gui;

import java.util.Arrays;

import org.Xwidgets.Styles;
import org.Xwidgets.XLabel;
import org.Xwidgets.XLayoutData;
import org.Xwidgets.event.XControlEvent;
import org.Xwidgets.event.XControlListener;
import org.apache.log4j.Logger;

final class FlowText extends Thread{
	private String text = "";
	private int speed = 140;
	private int visible = 0;
	private char[] visibleText;
	private int fontSize = 10;
	private XLabel flowText;
	private Logger logger;
	
	protected FlowText(XLayoutData d){
		logger = Logger.getLogger(getClass());
		flowText = MainGUI.mainGui.myGUIFactory.getLabel(MainGUI.mainGui.main, Styles.RIGHT);
		flowText.addControlListener(new XControlListener(){
			public void xControlMoved(XControlEvent event) {}
			public void xControlResized(XControlEvent event) {
				updateData();
			}
		});
		flowText.setLayoutData(d);
		start();
	}
	
	public void run(){
		while (!flowText.isDisposed()){
			if (visibleText != null){
				flow();
			}
			try {
				Thread.sleep(speed);
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
			}
		}
	}
	
	public void setSize(int x, int y){
		flowText.setSize(x, y);
		updateData();
	}
	
	public void flow(){
		prepareString();
		flowText.setText(String.copyValueOf(visibleText));
	}
	
	private void updateData(){
		int x = (int)flowText.getSize().getX();
		visibleText = new char[(int)(x/fontSize*2.5)];
		Arrays.fill(visibleText, ' ');
		visible = 0;
	}
	
	public void setFlowText(String text) {
		this.text = text;
		flowText.setToolTipText(text);
		Arrays.fill(visibleText, ' ');
		visible = 0;
	}
	
	private void prepareString(){
		if ((visible) > visibleText.length+text.length())
			visible = 0;
		if (text.length() > 0){
			for (int i = 0; i < visibleText.length-1; i++)
				visibleText[i] = visibleText[i+1];
			if (visible >= text.length() && visibleText.length > 0)
				visibleText[visibleText.length-1] = ' ';
			else if (visibleText.length > 0) visibleText[visibleText.length-1] = text.charAt(visible);
		}
		visible++;
	}
}
