/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.util.Iterator;
import java.util.Properties;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XComboBox;
import org.Xwidgets.XInputBox;
import org.Xwidgets.XInputValidator;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XMessageBox;
import org.Xwidgets.XTableItem;
import org.Xwidgets.XWidget;
import org.apache.log4j.Logger;
import org.x4x.core.Management;
import org.x4x.core.Title;
import org.x4x.plugins.OutputPlugin;
import org.x4x.plugins.Plugin;
import org.x4x.plugins.X4XPluginException;
import org.x4x.statics.StaticValues;

class GlobalEvents {
	private static WidgetFactory wt = MainGUI.mainGui.myGUIFactory;
	private static Management myManagement = Management.getInstance();
	private static Logger logger = Logger.getLogger(GlobalEvents.class);
	
	static void renamePlaylist(int items[]){
		if (items.length > 0) {
			MainGUI.mainGui.playlistPanel.playlistTable.setSelectedIndex(items[0]);
			XTableItem xti = MainGUI.mainGui.playlistPanel.playlistTable.getItemForIndex(items[0]);
			String nameold = xti.getText(1);
			
			if (xti != null){
				XInputBox newName = wt.getInputBox(null, Styles.QUESTION);
				newName.setInput(nameold);
				newName.setName((StaticValues.getLangString("renamePlaylist")));
				newName.setText((StaticValues.getLangString("renamePlaylist")));
				newName.setInputValidator(new XInputValidator(){
					public String isValid(String text) {
						if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
						if (text.length() < 3) return StaticValues.getLangString("tooShort");
						return null;
					}
				});
				newName.open();
				
				if (newName.getButton() == Styles.OK){
					String name = newName.getInput();
					myManagement.mlibRenamePlaylist(nameold,name);
					xti.setText(1, name);
				}			
			}
		}
	}
	
	static void savePlaylist(){
		XInputBox in = wt.getInputBox(null, Styles.QUESTION);
		in.setName(StaticValues.getLangString("savePlaylistAs"));
		in.setText(StaticValues.getLangString("savePlaylistAs"));
		in.setInputValidator(new XInputValidator(){
			public String isValid(String text) {
				if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
				if (text.length() < 3) return StaticValues.getLangString("tooShort");
				return null;
			}
		});
		in.open();
		if (in.getButton() == Styles.OK){
			myManagement.mlibSaveCurrentPlaylist(in.getInput());
			myManagement.mlibGetPlaylists();
		}
	}
	
	static void showTitleInfos(Title t){
		buildPropGUI(t);
	}
	
	static void searchMlib(){
		String query = "WHERE key";
		String sep = "=";
		String value = MainGUI.mainGui.mlibPanel.textSearch.getText().replaceAll("\\*", "%");
		if (getTextFromComboIndex(MainGUI.mainGui.mlibPanel.searchBox).equals("ownQuery"))
			query = value;
		else {
			if (!MainGUI.mainGui.mlibPanel.strict.isChecked()){
				sep = " LIKE ";
				if (value.indexOf("%") == -1)
					value = "%" + value + "%";
			}
			query += "='" + getTextFromComboIndex(MainGUI.mainGui.mlibPanel.searchBox) + "'" + 
			" AND value" + sep + "'" + value + "'";
		}
		myManagement.mlibSelect(query, true);
	}
	
	static void browseMlib(){
		String query = "WHERE key='" + 
		getTextFromComboIndex(MainGUI.mainGui.mlibPanel.browseBox) + "'";
			myManagement.mlibSelect(query, false);
	}
	
	static void handleMlibBrowsing(int items[]){
		if (items.length > 0){
			String query = "WHERE key='" + getTextFromComboIndex(MainGUI.mainGui.mlibPanel.browseBox) + 
			"' AND value='" + MainGUI.mainGui.mlibPanel.browseTable.getItemForIndex(items[0]).getText(0) + "'";
			myManagement.mlibSelect(query, true);
			MainGUI.mainGui.mlibPanel.textSearch.setText(
					MainGUI.mainGui.mlibPanel.browseTable.getItemForIndex(items[0]).getText(0));
			MainGUI.mainGui.mlibPanel.strict.setChecked(true);
		}
	}
	
	static void deletePlaylistItems(int items[]){
		if (items.length > 0){
			XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
			box.setText(StaticValues.getLangString("reallyDeleteItems"));
			box.open();
			if (box.getButton() == Styles.OK){
				long indices[] = new long[items.length];
				for ( int i = 0; i < items.length; i++)
					indices[i] = items[i];
				myManagement.removeTitlesFromPlaylist(indices);
			}
		}
	}
	
	static void deletePlaylists(int items[]){
		if ( items.length > 0){
			XMessageBox question = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
			question.setText(StaticValues.getLangString("reallyDeletePlaylist"));
			question.open();
			if (question.getButton() == Styles.OK){
				for ( int i = 0; i < items.length; i++){
					myManagement.mlibRemovePlaylist(MainGUI.mainGui.playlistPanel.
							playlistTable.getItemForIndex(items[i]).getText(1));
				}
				myManagement.mlibGetPlaylists();
			}
		}
	}
	
	/**
	 * Fill Property gui with title information from Title t, set t as current and open the frame
	 * 
	 * @param t
	 */
	private static void buildPropGUI(Title t) {
		int min, sec;
		
		if (t != null && (MainGUI.mainGui.propGui != null)) {
			try {
				MainGUI.mainGui.propGui.textTitle.setText(t.getAttribute("title"));
				MainGUI.mainGui.propGui.textArtist.setText(t.getAttribute("artist"));
				MainGUI.mainGui.propGui.textAlbum.setText(t.getAttribute("album"));
				MainGUI.mainGui.propGui.textGenre.setText(t.getAttribute("genre"));
				MainGUI.mainGui.propGui.textURL.setText(t.getAttribute("url"));
				MainGUI.mainGui.propGui.textDate.setText(t.getAttribute("date"));
				MainGUI.mainGui.propGui.textComment.setText(t.getAttribute("comment"));
				MainGUI.mainGui.propGui.labelFileType.setText(StaticValues.getLangString("fileType") + ": " + t.getAttribute("mime"));
				
				MainGUI.mainGui.propGui.labelID.setText("ID: " + t.getID());
				min = t.getLength()/60;
				sec = t.getLength()%60;
				MainGUI.mainGui.propGui.labelLength.setText(StaticValues.getLangString("length") + ": " + min + ":" + ((sec<10)?"0":"") + sec);		
				MainGUI.mainGui.propGui.labelBitRate.setText(StaticValues.getLangString("bitRate") + ": " + Integer.parseInt(t.getAttribute("bitrate"))/1000 + " kbit/s");
				MainGUI.mainGui.propGui.labelSampleRate.setText(StaticValues.getLangString("sampleRate") + ": " + t.getAttribute("samplerate") + " Hz");
			} catch (Exception e){
				logger.debug(e.getMessage(), e);
			}
			finally {
				MainGUI.mainGui.propGui.open(t);
			}
		}
	}
	
	/**
	 * used to get a "key" from a given Combobox. Useful in some Listeners for building Mlib queries
	 * 
	 * @param box		The XComboBox where you want to get your key from
	 * @return			the key, "" otherwise
	 */
	static String getTextFromComboIndex(XComboBox box){
		int index = box.getSelectedIndex();
		if (box.getRealObject().equals(MainGUI.mainGui.mlibPanel.searchBox.getRealObject())){
			switch (index){
				case 0:		return "artist";
				case 1:		return "title";
				case 2:		return "album";
				case 3:		return "all";
				case 4:		return "ownQuery";
				default:	return "";
			}
		}
		else if (box.getRealObject().equals(MainGUI.mainGui.mlibPanel.browseBox.getRealObject())){
			switch (index){
				case 0:		return "artist";
				case 1:		return "album";
				case 2:		return "genre";
				default:	return "";
			}
		}
		return "";
	}
	
	protected static void activatePlugin(String name) throws X4XPluginException{
		String plugConf[] = StaticValues.getKeys();
		Properties p = new Properties();
		for (int i = 0; i < plugConf.length; i++){
			if (plugConf[i].startsWith(name.toLowerCase() + ".")){
				p.setProperty(plugConf[i].replaceFirst(name.toLowerCase() + ".", ""), 
						StaticValues.getSetting(plugConf[i]));
			}
		}
		MainGUI.mainGui.pc.activatePlugin(name, p);
		Plugin pl = MainGUI.mainGui.pc.getActivePlugin(name);
		p = pl.getSettings();
		for (Iterator iter = p.keySet().iterator(); iter.hasNext();){
			String key = ""+iter.next();
			StaticValues.setSetting(name.toLowerCase() + "." + key, 
					p.getProperty(key));
		}
		if (MainGUI.mainGui.prefGui != null)
			MainGUI.mainGui.prefGui.update();
		if (pl instanceof OutputPlugin){
			XMenuItem it = MainGUI.mainGui.myGUIFactory.getMenuItem(
					MainGUI.mainGui.playlistPanel.menu);
			it.setText("*Plugin* " + name);
			it.addSelectionListener(SelectionListener.getInstance());
			MainGUI.mainGui.pluginMenuItems.add(it);
		}
		else {
			MainGUI.mainGui.pc.runPlugin(MainGUI.mainGui.pc.getActivePlugin(name));
		}
	}
	
	protected static void deactivatePlugin(String name) throws X4XPluginException{
		Class cl = MainGUI.mainGui.pc.getPluginclass(name);
		MainGUI.mainGui.pc.deactivatePlugin(name);
		if (cl.equals(OutputPlugin.class)){
			for (Iterator it = MainGUI.mainGui.pluginMenuItems.iterator(); it.hasNext();){
				XMenuItem xit = (XMenuItem)it.next();
				if (xit.getText().equals("*Plugin* " + name)){
					it.remove();
					xit.dispose();
				}
			}
		}
	}

	protected static boolean checkSource(Object source, XWidget compare){
		if (source != null && compare != null && source.equals(compare.getRealObject())) return true;
		return false;
	}
}
