/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Copyright Georg Schild (dangertools@gmail.com), Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.net.MalformedURLException;
import java.util.Timer;
import java.util.TimerTask;

import jcifs.smb.SmbFile;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XInputBox;
import org.Xwidgets.XInputValidator;
import org.Xwidgets.XMessageBox;
import org.Xwidgets.XTableItem;
import org.Xwidgets.event.XKeyListener;
import org.Xwidgets.event.XKeyEvent;
import org.apache.log4j.Logger;
import org.x4x.core.Management;
import org.x4x.statics.StaticValues;


/**
 * Packagekeylistener used for some global key-events
 */

class KeyListener extends XKeyListener {
	private Management myManagement;
	private Logger logger;
	private static KeyListener instance;
	private WidgetFactory wt;
	
	private KeyListener(){
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
		wt = MainGUI.mainGui.myGUIFactory;
	}
	
	/**
	 * Get a KeyListener. You usually want to use that to add a listener of this class to your control
	 * 
	 * @return		a singleton keylistener
	 */
	public static KeyListener getInstance(){
		if (instance == null)
			instance = new KeyListener();
		return instance;
	}
	
	/**
	 * Handle a keyPressed Event
	 */
	public void xKeyPressed(XKeyEvent arg0) {
		if ((arg0.keyCode == 's' || arg0.keyCode == 'S') && 
				(arg0.stateMask&XKeyEvent.CTRL) == XKeyEvent.CTRL &&
				GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.playlistPanel.table)){
			GlobalEvents.savePlaylist();
		}
		else if (arg0.keyCode == (char)13){
			if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.mlibPanel.textSearch)){
				GlobalEvents.searchMlib();
			}
			else if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.CIFSBrowser.adress)){
				try {
					String line = MainGUI.mainGui.CIFSBrowser.adress.getText();
					if (!line.endsWith("/"))
						line += "/";
					SmbFile tmp = new SmbFile("smb:/" + line);
					MainGUI.mainGui.CIFSBrowser.fillTable(tmp);
				} catch (MalformedURLException e) {
					logger.debug(e.getMessage(), e);
				}
			}
		}
		else if (arg0.keyCode == (char)127){
			if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.playlistPanel.table)){
				GlobalEvents.deletePlaylistItems(MainGUI.mainGui.playlistPanel.table.getSelectedIndices());
			}
			else if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.playlistPanel.playlistTable)){
				GlobalEvents.deletePlaylists(MainGUI.mainGui.playlistPanel.playlistTable.getSelectedIndices());
			}
		}
		else if (arg0.keyCode == XKeyEvent.F2){
			if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.playlistPanel.playlistTable)){
				GlobalEvents.renamePlaylist(MainGUI.mainGui.playlistPanel.playlistTable.getSelectedIndices());
			}
			else if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.playlistPanel.table)){
				int items[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
				if (items.length > 0)
					GlobalEvents.showTitleInfos(myManagement.getCurrentPlaylist().getTitleForIndex(items[0]));
			}
			else if (GlobalEvents.checkSource(arg0.getSource(), MainGUI.mainGui.mlibPanel.searchTable)){
				int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
				if (items.length > 0){
					final long id = Long.parseLong(
							MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex(items[0]).getText(0));
					if (!myManagement.isTitleInCache(id))
						new Timer().schedule(new TimerTask(){
							public void run(){
								GlobalEvents.showTitleInfos(myManagement.getTitleForID(id));
							}
						}, 3000);
					else GlobalEvents.showTitleInfos(myManagement.getTitleForID(id));
				}
			}
		}
	}
	
	/**
	 * Handle a KeyReleased Event
	 */
	public void xKeyReleased(XKeyEvent arg0) {}
	
}
