/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * 
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.awt.Toolkit;
import java.io.File;
import java.io.FileFilter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.swing.JOptionPane;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XGridData;
import org.Xwidgets.XGridLayout;
import org.Xwidgets.XMainFrame;
import org.Xwidgets.XPanel;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XMenu;
import org.Xwidgets.XMenuBar;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XMessageBox;
import org.Xwidgets.XSlider;
import org.Xwidgets.XSplitPane;
import org.Xwidgets.XSystemTray;
import org.Xwidgets.XTabbedPane;
import org.Xwidgets.SWT.SWTFactory;
import org.apache.log4j.Logger;
import org.jdom.DataConversionException;
import org.jdom.Element;
import org.x4x.core.Management;
import org.x4x.gui.models.DNA;
import org.x4x.plugins.PluginController;
import org.x4x.statics.StaticValues;


/**
 * Main class of X4X. This is the only class with a main() Method. It just builds and defines the UI-Elements,
 * some things at startup are done too. No computation happens here
 */
public final class MainGUI {
	protected ButtonBar buttonPanel;
	protected PlaylistPanel playlistPanel;
	protected EQPanel eqPanel;
	protected NetworkBrowser CIFSBrowser;
	protected XSlider volume, playtime;
	protected XTabbedPane tabbedPane;
	private Logger logger;
	protected XMainFrame myFrame;
	protected XFrame lists, eqWindow;
	protected WidgetFactory myGUIFactory;
	private DNA dna;
	protected FlowText flow;
	protected PropGUI propGui;
	protected PrefGUI prefGui;
	protected MlibPanel mlibPanel;
	protected XMenuItem menuExit, menuPrefs, menuMlib, aboutEntry, helpEntry, eqEntry, movieEntry, 
			trayExit, trayAbout, trayPlay, trayStop, trayPause, trayNext, trayPrev;
	protected XMenuBar menuBar;
	protected XGridLayout layoutWithBorders, layoutWithoutBorders, headerLayout;
	protected XGridData layoutDataFillHoriz, layoutDataFillBoth;
	private XGridData content, vol, prog, currentPlaying, labels;
	protected XGridData buttons;
	protected boolean tabbed, external, packed;
	protected XLabel status, currentVolume, currentProgress;
	private Element me;
	protected XSystemTray tray;
	protected String iconPrefix;
	protected PluginController pc;
	protected List pluginMenuItems;
	protected XMenu plugins;
	protected XPanel main, statusLine;
	protected XLabel shuffle, repeatAll, repeatOne;
	
	protected static MainGUI mainGui;
	
	public static final String version = "0.2.1-Carol";
	
	/**
	 * construct and start the mainGUI
	 * @param args 
	 */
	public MainGUI(String[] args){
		logger = Logger.getLogger(getClass());
		initialize();
		new Errorhandler();
		iconPrefix = StaticValues.getSetting("iconDir") + StaticValues.getSetting("iconstyle") + StaticValues.getSetting("deli");
		mainGui = this;
		logger.info("==== " + Calendar.getInstance(Locale.getDefault()).getTime() + " ====");
		logger.info("X4X version " + version + " started");
		dna = new DNA();
		if (StaticValues.getSetting("gui").equals("swt")){
			try {
				myGUIFactory = SWTFactory.getInstance();
			} catch (Throwable e) {
				JOptionPane.showMessageDialog(null, "Unable to find SWT Library.\nPlease "+
						"(re-)install SWT", "SWT missing", JOptionPane.ERROR_MESSAGE);
				System.exit(1);
			}
		}
		layoutWithBorders = myGUIFactory.getGridLayout();
		layoutWithBorders.makeColumnsEqualWidth(false);
		layoutWithBorders.setColumnNums(1);
		layoutWithBorders.setMarginHeight(50);
		layoutWithBorders.setMarginWidth(30);
		layoutWithBorders.setVerticalSpacing(10);

		layoutWithoutBorders = myGUIFactory.getGridLayout();
		layoutWithoutBorders.setColumnNums(1);
		layoutWithoutBorders.makeColumnsEqualWidth(false);
		layoutWithoutBorders.setVerticalSpacing(10);
		
		layoutDataFillHoriz = myGUIFactory.getGridData();
		layoutDataFillHoriz.setExcessHorizontalSpace(true);
		layoutDataFillHoriz.setHeightHint(27);
		layoutDataFillHoriz.setHorizontalAlignment(Styles.FILL);
		
		layoutDataFillBoth = myGUIFactory.getGridData();
		layoutDataFillBoth.setHorizontalAlignment(Styles.FILL);
		layoutDataFillBoth.setVerticalAlignment(Styles.FILL);
		layoutDataFillBoth.setExcessHorizontalSpace(true);
		layoutDataFillBoth.setExcessVerticalSpace(true);

		XGridLayout mainGUILayout = myGUIFactory.getGridLayout();
		mainGUILayout.makeColumnsEqualWidth(false);
		mainGUILayout.setColumnNums(8);
		mainGUILayout.setMarginWidth(30);
		
		content = myGUIFactory.getGridData();
		content.setExcessHorizontalSpace(true);
		content.setExcessVerticalSpace(true);
		content.setHorizontalSpan(8);
		content.setHorizontalAlignment(Styles.FILL);
		content.setVerticalAlignment(Styles.FILL);
		
		vol = myGUIFactory.getGridData();
		vol.setExcessHorizontalSpace(true);
		vol.setHorizontalSpan(6);
		vol.setHorizontalAlignment(Styles.FILL);
		
		prog = myGUIFactory.getGridData();
		prog.setExcessHorizontalSpace(true);
		prog.setHorizontalSpan(6);
		prog.setHorizontalAlignment(Styles.FILL);
		
		buttons = myGUIFactory.getGridData();
		buttons.setExcessHorizontalSpace(true);
		buttons.setHorizontalSpan(8);
		buttons.setHorizontalAlignment(Styles.FILL);
		
		currentPlaying = myGUIFactory.getGridData();
		currentPlaying.setExcessHorizontalSpace(true);
		currentPlaying.setHorizontalSpan(7);
		currentPlaying.setHorizontalAlignment(Styles.FILL);
		
		labels = myGUIFactory.getGridData();
		labels.setHorizontalSpan(1);
		labels.setWidthHint(120);

		myGUIFactory.setAppName("X4X");
		try {
			me = dna.getBaseElement("MainGUI");
			if (me == null){
				logger.fatal(StaticValues.getLangString("noMe"));
				XMessageBox error = myGUIFactory.getMessageBox(null, Styles.SIMPLE);
				error.setText(StaticValues.getLangString("noMe"));
				error.open();
				System.exit(1);
			}
			if (me.getAttribute("resizeable").getBooleanValue())
				myFrame = myGUIFactory.getMainFrame(Styles.RESIZE);
			else myFrame = myGUIFactory.getMainFrame(Styles.SIMPLE);
			
			myFrame.setLayout(myGUIFactory.getGridLayout());
			myFrame.setImage(iconPrefix + "trayIcon.png");
			
			main = myGUIFactory.getPanel(myFrame, Styles.SIMPLE);
			XGridData mainData = myGUIFactory.getGridData();
			mainData.setExcessHorizontalSpace(true);
			mainData.setExcessVerticalSpace(true);
			mainData.setHorizontalAlignment(Styles.FILL);
			mainData.setVerticalAlignment(Styles.FILL);
			main.setLayoutData(mainData);
			main.setLayout(mainGUILayout);
			
			pluginMenuItems = new ArrayList();
			
			XGridData imageData = myGUIFactory.getGridData();
			imageData.setHeightHint(25);
			imageData.setWidthHint(25);

			List masters = dna.getChildren(me);
			for (ListIterator it = masters.listIterator(); it.hasNext();){
				Element current = (Element)it.next();
				if (current.getName().equals("currentTitle")){
					flow = new FlowText(currentPlaying);
					repeatAll = myGUIFactory.getLabel(main, Styles.SIMPLE);
					repeatAll.setLayoutData(imageData);
					repeatAll.addMouseListener(MouseListener.getInstance());
				}
				else if (current.getName().equals("VolumeSlider")){
					currentVolume = myGUIFactory.getLabel(main, Styles.LEFT);
					currentVolume.setLayoutData(labels);
					volume = buildSlider(current);
					volume.setLayoutData(vol);
					volume.setMax(100);
					repeatOne = myGUIFactory.getLabel(main, Styles.SIMPLE);
					repeatOne.setLayoutData(imageData);
					repeatOne.addMouseListener(MouseListener.getInstance());
				}
				else if (current.getName().equals("PlaytimeSlider")){
					currentProgress = myGUIFactory.getLabel(main, Styles.LEFT);
					currentProgress.setLayoutData(labels);
					playtime = buildSlider(current);
					playtime.setLayoutData(prog);
					shuffle = myGUIFactory.getLabel(main, Styles.SIMPLE);
					shuffle.setLayoutData(imageData);
					shuffle.addMouseListener(MouseListener.getInstance());
				}
				else if (current.getName().equals("Lists")){
					buildPanels(dna.getChild(me, "Lists"));
				}
				else if (current.getName().equals("ButtonBar")){
					buttonPanel = ButtonBar.getButtonBar(main, current);
				}
				else {
					logger.debug(StaticValues.getLangString("unknownElement") + ": " + current.getName());
					XMessageBox error = myGUIFactory.getMessageBox(myFrame, Styles.OK|Styles.ERROR);
					error.setText(StaticValues.getLangString("unknownElement"));
					error.open();
				}
			}
			
			statusLine = myGUIFactory.getPanel(myFrame, Styles.BORDER);
			statusLine.setLayout(myGUIFactory.getGridLayout());
			
			status = myGUIFactory.getLabel(statusLine, Styles.LEFT);
			XGridData statusData = myGUIFactory.getGridData();
			statusData.setExcessHorizontalSpace(true);
			statusData.setHorizontalAlignment(Styles.FILL);
			status.setLayoutData(statusData);
			statusLine.setLayoutData(statusData);
			
			CIFSBrowser = new NetworkBrowser(layoutWithBorders, layoutDataFillHoriz, myGUIFactory);
			buildMenu();
			buildTray();
			initPlugins();
			
			myFrame.setText("X4X @" + StaticValues.getSetting("backend").toUpperCase());

			new UpdateListener(args);
			propGui = new PropGUI();
			prefGui = new PrefGUI();
		} catch (Throwable e){
			logger.debug(e.getMessage(), e);
		}
	}
	
	private void initPlugins() throws MalformedURLException {
		if (!StaticValues.getSetting("plugindir").equals("")){
			File plugdir = new File(StaticValues.getSetting("plugindir"));
			File plugs[] = plugdir.listFiles(new FileFilter(){
				public boolean accept(File arg0) {
					return arg0.getName().toLowerCase().endsWith(".jar");
				}
			});
			if (plugs != null && plugs.length > 0){
				URL urls[] = new URL[plugs.length];
				for (int i = 0; i < plugs.length; i++){
					urls[i] = plugs[i].toURL();
				}
				pc = new PluginController(urls, myGUIFactory, new PluginConnector());
				pc.addPluginListener(new PluginListener());
				if (urls.length > 0){
					myGUIFactory.getMenuSeparator(playlistPanel.menu);
					myGUIFactory.getMenuSeparator(mlibPanel.menuMedial);
					String actives[] = StaticValues.getSetting("activePlugins").split(";");
					for (int i = 0; i < actives.length; i++){
						if (!actives[i].equals("")){
							try {
								GlobalEvents.activatePlugin(actives[i]);
							} catch (Exception e){
								logger.debug(e.getMessage(), e);
							}
						}
					}
				}
			}
			else {
				pc = new PluginController(new URL[]{}, myGUIFactory, new PluginConnector());
			}
		}
		else pc = new PluginController(new URL[]{}, myGUIFactory, new PluginConnector());
	}
	
	public void open(){
		try {
			if (!external && tabbedPane != null){
				myFrame.setSize(me.getAttribute("width").getIntValue(), 
						me.getAttribute("height").getIntValue());
			}
			else {
				packed = true;
				myFrame.pack();
			}
			if (!external && tabbedPane == null)
				myFrame.setSize(Toolkit.getDefaultToolkit().getScreenSize().width-5,
						Toolkit.getDefaultToolkit().getScreenSize().height-5);
			myGUIFactory.run(this, getClass().getDeclaredMethod("finalizer", new Class[]{}), new Object[]{});
		} catch (DataConversionException e) {
			logger.debug(e.getMessage(), e);
		} catch (SecurityException e) {
			logger.debug(e.getMessage(), e);
		} catch (NoSuchMethodException e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	private void buildTray(){		
		tray = myGUIFactory.getSystemTray(Styles.NULL);
		tray.setName("X4X");
		tray.setTooltipText("X4X");
		tray.setImage(iconPrefix + "trayIcon.png");
		
		XMenu trayMenu = myGUIFactory.getDropDownMenu(myFrame);
		
		trayPlay = myGUIFactory.getMenuItem(trayMenu);
		trayPlay.setText(StaticValues.getLangString("PlayButton"));
		trayPlay.addSelectionListener(SelectionListener.getInstance());
		trayPlay.setImage(iconPrefix + "trayPlay.png");
		
		trayPause = myGUIFactory.getMenuItem(trayMenu);
		trayPause.setText(StaticValues.getLangString("PauseButton"));
		trayPause.addSelectionListener(SelectionListener.getInstance());
		trayPause.setImage(iconPrefix + "trayPause.png");
		
		trayStop = myGUIFactory.getMenuItem(trayMenu);
		trayStop.setText(StaticValues.getLangString("StopButton"));
		trayStop.addSelectionListener(SelectionListener.getInstance());
		trayStop.setImage(iconPrefix + "trayStop.png");
		
		trayNext = myGUIFactory.getMenuItem(trayMenu);
		trayNext.setText(StaticValues.getLangString("NextButton"));
		trayNext.addSelectionListener(SelectionListener.getInstance());
		trayNext.setImage(iconPrefix + "trayNext.png");
		
		trayPrev = myGUIFactory.getMenuItem(trayMenu);
		trayPrev.setText(StaticValues.getLangString("PrevButton"));
		trayPrev.addSelectionListener(SelectionListener.getInstance());
		trayPrev.setImage(iconPrefix + "trayPrev.png");
		
		myGUIFactory.getMenuSeparator(trayMenu);
		
		trayAbout = myGUIFactory.getMenuItem(trayMenu);
		trayAbout.setText(StaticValues.getLangString("aboutEntry"));
		trayAbout.addSelectionListener(SelectionListener.getInstance());
		trayAbout.setImage(iconPrefix + "trayAbout.png");
		
		trayExit = myGUIFactory.getMenuItem(trayMenu);
		trayExit.setText(StaticValues.getLangString("exitEntry"));
		trayExit.addSelectionListener(SelectionListener.getInstance());
		trayExit.setImage(iconPrefix + "trayExit.png");
		
		tray.setMenu(trayMenu);
		// Current XWidgets had a failure there
		//tray.addSelectionListener(SelectionListener.getInstance());
	}

	/**
	 * Build menu
	 */
	private void buildMenu() {
		menuBar = myGUIFactory.getMenuBar(myFrame);
		XMenu file = myGUIFactory.getMenu(myFrame);
		menuBar.addMenu(file, StaticValues.getLangString("fileMenu"));
		plugins = myGUIFactory.getMenu(myFrame);
		menuBar.addMenu(plugins, "Plugins");
		XMenu help = myGUIFactory.getMenu(myFrame);
		menuBar.addMenu(help, StaticValues.getLangString("helpMenu"));
		
		if (external){
			menuMlib = myGUIFactory.getMenuItem(file);
			menuMlib.setText(StaticValues.getLangString("lists"));
			menuMlib.addSelectionListener(SelectionListener.getInstance());
			menuMlib.setImage(iconPrefix + "trayLists.png");
		}
		
		menuPrefs = myGUIFactory.getMenuItem(file);
		menuPrefs.setText(StaticValues.getLangString("preferences"));
		menuPrefs.addSelectionListener(SelectionListener.getInstance());
		menuPrefs.setImage(iconPrefix + "trayPrefs.png");
		
		eqEntry = myGUIFactory.getMenuItem(file);
		eqEntry.setText(StaticValues.getLangString("eqEntry"));
		eqEntry.addSelectionListener(SelectionListener.getInstance());
		eqEntry.setImage(iconPrefix + "trayEQ.png");
		
		myGUIFactory.getMenuSeparator(file);
		
		/* TODO: mark a big TODO here! This videosupport is just temporary
		 * and hopefully will get better implemented soon as it's really ugly
		 * atm ;)
		 */
		
		/*movieEntry = myGUIFactory.getMenuItem(file);
		movieEntry.setText("Play Video");
		movieEntry.addSelectionListener(SelectionListener.getInstance());
		movieEntry.setImage(iconPrefix + "trayMovie.png");
		
		myGUIFactory.getMenuSeparator(file);*/
		
		menuExit = myGUIFactory.getMenuItem(file);
		menuExit.setText(StaticValues.getLangString("exitEntry"));
		menuExit.addSelectionListener(SelectionListener.getInstance());
		menuExit.setImage(iconPrefix + "trayExit.png");
		
		helpEntry = myGUIFactory.getMenuItem(help);
		helpEntry.setText(StaticValues.getLangString("helpEntry"));
		helpEntry.addSelectionListener(SelectionListener.getInstance());
		helpEntry.setImage(iconPrefix + "trayHelp.png");
		
		myGUIFactory.getMenuSeparator(help);
		
		aboutEntry = myGUIFactory.getMenuItem(help);
		aboutEntry.setText(StaticValues.getLangString("aboutEntry"));
		aboutEntry.addSelectionListener(SelectionListener.getInstance());
		aboutEntry.setImage(iconPrefix + "trayAbout.png");
		
		myFrame.setMenuBar(menuBar);
	}
	
	/**
	 * Build slider from Element
	 * 
	 * @param el		jdom Element
	 * @return			XSlider
	 */
	private XSlider buildSlider(Element el) {
		XSlider s = myGUIFactory.getSlider(main, Styles.HORIZONTAL);
		s.addMouseListener(MouseListener.getInstance());
		return s;
	}
	
	/**
	 * build panels (playlist/playlists/mlib-search/mlib-browse)
	 * 
	 * @param dnaLists		jdom Element describing look of lists
	 */
	private void buildPanels(Element dnaLists) {
		tabbed = true;
		external = true;
		try {
			tabbed = dnaLists.getAttribute("tabbed").getBooleanValue();
			external = dnaLists.getAttribute("external").getBooleanValue();
		} catch (DataConversionException e1) {
			logger.debug(e1.getMessage(), e1);
		}
		try {
			XPanel dummy = main;
			if (external){
				lists = myGUIFactory.getFrame(myFrame, Styles.RESIZE);
				lists.setSize(600, 600);
				lists.disposeOnClose(false);
				lists.setImage(iconPrefix + "trayLists.png");
				lists.setText(StaticValues.getLangString("lists"));
				lists.setLayout(layoutWithoutBorders);
				lists.setLayoutData(layoutDataFillHoriz);
				dummy = lists;
			}
			if(tabbed) {
				playlistPanel = PlaylistPanel.getPlaylistPanel(dummy, dna.getChild(dnaLists, "PlaylistPanel"), 
						layoutWithoutBorders, layoutDataFillHoriz);
				mlibPanel = MlibPanel.getInstance(dummy, dna.getChild(dnaLists, "MedialibPanel"), 
						layoutWithoutBorders, layoutDataFillHoriz);
				tabbedPane = myGUIFactory.getTabbedPane(dummy, Styles.SIMPLE);
				tabbedPane.setLayout(layoutWithoutBorders);
				tabbedPane.setLayoutData(content);
				tabbedPane.addSelectionListener(SelectionListener.getInstance());
				tabbedPane.addItem(playlistPanel.panel,StaticValues.getLangString("playlist"), 
						StaticValues.getLangString("playlistHeader"));
				tabbedPane.addItem(playlistPanel.playlistsPanel,StaticValues.getLangString("playlists"),
						StaticValues.getLangString("playlistsHeader"));
				tabbedPane.addItem(mlibPanel.browsePanel, StaticValues.getLangString("mlibBrowse"), 
						StaticValues.getLangString("mlibBrowseHeader"));
				tabbedPane.addItem(mlibPanel.searchPanel, StaticValues.getLangString("mlibSearch"), 
						StaticValues.getLangString("mlibSearchHeader"));
			}
			else {
				XSplitPane mlibSplit, playlistSplit;
				playlistPanel = PlaylistPanel.getPlaylistPanel(dummy, dna.getChild(dnaLists, "PlaylistPanel"), 
						layoutWithoutBorders, layoutDataFillHoriz);
				mlibPanel = MlibPanel.getInstance(dummy, dna.getChild(dnaLists, "MedialibPanel"), 
						layoutWithoutBorders, layoutDataFillHoriz);
				XSplitPane splitPane = myGUIFactory.getSplitPane(dummy, Styles.VERTICAL);
				splitPane.setLayout(layoutWithoutBorders);
				splitPane.setLayoutData(content);
				splitPane.setItems((mlibSplit = myGUIFactory.getSplitPane(splitPane, Styles.HORIZONTAL)), 
						(playlistSplit = myGUIFactory.getSplitPane(splitPane, Styles.HORIZONTAL)));
				mlibSplit.setLayout(layoutWithoutBorders);
				mlibSplit.setLayoutData(layoutDataFillBoth);
				playlistSplit.setLayout(layoutWithoutBorders);
				playlistSplit.setLayoutData(layoutDataFillBoth);
				mlibSplit.setItems(mlibPanel.browsePanel, mlibPanel.searchPanel);
				playlistSplit.setItems(playlistPanel.playlistsPanel, playlistPanel.panel);
			}
		} catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Only method which is used elsewhere too. A listener could call it got get a really actual EQPanel
	 *
	 */
	protected void buildEQ(){
		int val = 0;
		while(Management.getInstance().getEQ(val) > -51)
			val++;
		if (eqWindow == null || eqPanel.eqCount != val){
			if (eqWindow != null) eqWindow.dispose();
			eqWindow = myGUIFactory.getFrame(myFrame, Styles.SIMPLE);
			eqWindow.setSize(400, 150);
			eqWindow.setText(StaticValues.getLangString("eq"));
			eqWindow.disposeOnClose(false);
			eqWindow.setImage(iconPrefix + "trayEQ.png");
			eqWindow.setLayout(layoutWithoutBorders);
			eqPanel = EQPanel.getEQPanel(eqWindow, null, val);
			eqWindow.pack();
		}
	}
	
	/**
	 * Called on finalization from the WidgetFactory
	 *
	 */
	public void finalizer(){
		List actives = pc.spinDown();
		String act = "";
		int i = 0;
		for (Iterator it = actives.iterator(); it.hasNext();){
			String name = ""+it.next();
			act += name;
			if (i++ < actives.size()-1)
				act += ";";
		}
		StaticValues.setSetting("activePlugins", act);
		
		StaticValues.finish();
		Management.getInstance().spinDown();
		System.exit(0);
	}
	
	/**
	 * Called on first startup to copy dirs etc.
	 */
	private void initialize() {
		File confDir = new File(StaticValues.getSetting("prefdir"));
		//if (!confDir.exists()){
			Logger.getLogger(MainGUI.class).debug("Creating preferences directory at " + confDir);
			InputStream res = getClass().getResourceAsStream("/xmms2.zip");
			if (res != null){
				ZipInputStream in = new ZipInputStream(res);
				ZipEntry next = null;
				try {
					while((next = in.getNextEntry()) != null){
						if (next.isDirectory())
							new File(confDir, next.getName()).mkdirs();
						else {
							FileOutputStream out = new FileOutputStream(new File(confDir, next.getName()));
							byte[] buf = new byte[1024];
							int len;
							while ((len = in.read(buf)) > 0) {
								out.write(buf, 0, len);
							}
							out.flush();
							out.close();
						}
					}
					in.close();
				} catch (IOException e) {
					logger.debug(e.getMessage(), e);
				}
			}
			else {
				logger.warn("We are missing xmms2.zip which should be part of X4X.jar.\n" +
						"This means you are running X4X not as it is intended to be run\n" +
						"If you miss icons and other things try to copy .xmms2/clients/x4x " +
						"from some other location");
			}
		//}
	}
}
