/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XCheckBox;
import org.Xwidgets.XComboBox;
import org.Xwidgets.XGridData;
import org.Xwidgets.XGridLayout;
import org.Xwidgets.XLabel;
import org.Xwidgets.XLayout;
import org.Xwidgets.XLayoutData;
import org.Xwidgets.XMenu;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTable;
import org.Xwidgets.XTextField;
import org.Xwidgets.XWidgetsException;
import org.apache.log4j.Logger;
import org.jdom.Element;
import org.x4x.statics.StaticValues;


/**
 * Class represents the Medialib part of the UI. It provides to panels, search- and browsePanel which can be used elsewhere.
 * 
 */
class MlibPanel {
	protected XTable searchTable, browseTable;
	protected XTextField textSearch;
	protected XPanel searchPanel, browsePanel;
	protected XComboBox searchBox, browseBox;
	protected XButton searchButton;
	protected XCheckBox strict;
	protected XMenu menuMedial, browseMenu;
	protected XMenuItem addToPlaylist, addEntryMedialib, addRandomEntries, addDirectory,
		searchFor, browseNetwork, titleInfo, refreshMlib;
	protected XLabel preSearchbox;
	private XPanel parent;
	private Element mlib;
	private WidgetFactory wt;
	private Logger logger;
	private static MlibPanel instance;
	protected XLayout layout, rows;
	protected XLayoutData layoutData, rowsLayout, field;
	
	private MlibPanel(XPanel parent, Element mlib, XLayout panelLayout, XLayoutData layoutData) {
		logger = Logger.getLogger(getClass());
		this.parent = parent;
		this.mlib = mlib;
		layout = panelLayout;
		this.layoutData = layoutData;
		wt = MainGUI.mainGui.myGUIFactory;
		buildMe();
	}
	
	/**
	 * Get and/or build the panels
	 * 
	 * @param parent			parent element of the panels, maybe a dummy
	 * @param mlib				Element containing the builddata for both panels
	 * @param panelLayout		layoutdata (with/without borders)
	 * @param layoutData		how to fill the layout
	 * @return					return a MlibPanel
	 */
	public static MlibPanel getInstance(XPanel parent, Element mlib, XLayout panelLayout, XLayoutData layoutData){
		if (instance == null)
			instance = new MlibPanel(parent, mlib, panelLayout, layoutData);
		return instance;
	}
	
	/**
	 * Set Layout(data) for later use
	 *
	 */
	private void setLayout(){
		rows = wt.getGridLayout();
		((XGridLayout)rows).makeColumnsEqualWidth(false);
		((XGridLayout)rows).setColumnNums(7);
		rowsLayout = wt.getGridData();
		((XGridData)rowsLayout).setHorizontalAlignment(Styles.FILL);
		((XGridData)rowsLayout).setHeightHint(35);
		field = wt.getGridData();
		((XGridData)field).setExcessHorizontalSpace(true);
		((XGridData)field).setHorizontalAlignment(Styles.FILL);
		((XGridData)field).setHorizontalSpan(3);
	}
	
	/**
	 * Called by Constructor to build Elements
	 *
	 */
	private void buildMe(){
		try {
			setLayout();
			buildSearchPanel();
			buildBrowsePanel();
		} catch (XWidgetsException e) {
			logger.debug(e.getMessage(), e);
		}
	}
	
	/**
	 * Constructs the Searchpanel
	 * @throws XWidgetsException		if layout is not ok
	 */
	private void buildSearchPanel() throws XWidgetsException {
		searchPanel = wt.getPanel(parent, Styles.BORDER);
		searchPanel.setLayout(layout);
		
		XLabel header = wt.getLabel(searchPanel, Styles.CENTER);
		header.setText(StaticValues.getLangString("mlibSearch").toUpperCase());
		header.setLayoutData(layoutData);
		header.setToolTipText(StaticValues.getLangString("mlibSearchHeader"));
		
		searchBox = wt.getComboBox(searchPanel,Styles.BORDER|Styles.READ_ONLY);
		searchBox.setData(new String[]{StaticValues.getLangString("artist"),StaticValues.getLangString("title"),
				StaticValues.getLangString("album"),StaticValues.getLangString("allTitles")});
		searchBox.setLayoutData(layoutData);
		searchBox.addSelectionListener(SelectionListener.getInstance());
		
		XPanel tmp = wt.getPanel(searchPanel, Styles.SIMPLE);
		tmp.setLayout(rows);
		tmp.setLayoutData(rowsLayout);
		
		preSearchbox = wt.getLabel(tmp, Styles.LEFT);
		preSearchbox.setText(StaticValues.getLangString("searchFor"));
		preSearchbox.setLayoutData(wt.getGridData());
		
		textSearch = wt.getTextField(tmp, Styles.BORDER|Styles.SINGLE);
		textSearch.setLayoutData(field);
		textSearch.addKeyListener(KeyListener.getInstance());
		
		strict = wt.getCheckBox(tmp, Styles.SIMPLE);
		strict.setText(StaticValues.getLangString("strict"));
		strict.setLayoutData(wt.getGridData());
		strict.setToolTipText(StaticValues.getLangString("exactMeans"));
		
		XGridData buttonData = wt.getGridData();
		buttonData.setHorizontalSpan(2);
		buttonData.setWidthHint(120);
		searchButton = wt.getButton(tmp, Styles.CENTER);
		searchButton.setText(StaticValues.getLangString("search"));
		searchButton.setLayoutData(buttonData);
		searchButton.addMouseListener(MouseListener.getInstance());
		
		searchTable = wt.getTable(searchPanel, Styles.BORDER|Styles.MULTI);
		searchTable.setColumnNames(new String[]{StaticValues.getLangString("id"),
			StaticValues.getLangString("artist"),StaticValues.getLangString("title"),
			StaticValues.getLangString("album"),StaticValues.getLangString("genre")});
		searchTable.setColumnWidths(new int[]{45,135,250,135,135});
		searchTable.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
		searchTable.addMouseListener(MouseListener.getInstance());
		searchTable.addKeyListener(KeyListener.getInstance());
		
		menuMedial = wt.getDropDownMenu(searchTable);
		menuMedial.addMenuListener(MenuListener.getInstance());
		addToPlaylist = wt.getMenuItem(menuMedial);
		addToPlaylist.setText(StaticValues.getLangString("addToPlaylist"));
		addToPlaylist.addSelectionListener(SelectionListener.getInstance());
		titleInfo = wt.getMenuItem(menuMedial);
		titleInfo.setText(StaticValues.getLangString("showPlaylistEntry"));
		titleInfo.addSelectionListener(SelectionListener.getInstance());
		searchTable.setMenu(menuMedial);
	}
	
	/**
	 * Constructs Browsepanel
	 * 
	 * @throws XWidgetsException		if layout is not ok
	 */
	private void buildBrowsePanel() throws XWidgetsException {
		browsePanel = wt.getPanel(parent, Styles.BORDER);
		browsePanel.setLayout(layout);
		
		XLabel header = wt.getLabel(browsePanel, Styles.CENTER);
		header.setText(StaticValues.getLangString("mlibBrowse").toUpperCase());
		header.setLayoutData(layoutData);
		header.setToolTipText(StaticValues.getLangString("mlibBrowseHeader"));
		
		browseBox = wt.getComboBox(browsePanel, Styles.BORDER|Styles.READ_ONLY);
		browseBox.setData(new String[]{StaticValues.getLangString("artist"),
				StaticValues.getLangString("album"),StaticValues.getLangString("genre")});
		browseBox.setLayoutData(layoutData);
		browseBox.addSelectionListener(SelectionListener.getInstance());
		
		browseTable = wt.getTable(browsePanel, Styles.BORDER|Styles.MULTI);
		browseTable.setColumnNames(new String[]{
				StaticValues.getLangString("contentBrowse")});
		browseTable.setColumnWidths(new int[]{300});
		browseTable.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
		browseTable.addMouseListener(MouseListener.getInstance());
		
		browseMenu = wt.getDropDownMenu(browsePanel);
		addEntryMedialib = wt.getMenuItem(browseMenu);
		addEntryMedialib.setText(StaticValues.getLangString("mlibAdd"));
		addEntryMedialib.addSelectionListener(SelectionListener.getInstance());
		addDirectory = wt.getMenuItem(browseMenu);
		addDirectory.setText(StaticValues.getLangString("mlibAddDir"));
		addDirectory.addSelectionListener(SelectionListener.getInstance());
		browseNetwork = wt.getMenuItem(browseMenu);
		browseNetwork.setText(StaticValues.getLangString("mlibBrowseNetwork"));
		browseNetwork.addSelectionListener(SelectionListener.getInstance());
		searchFor = wt.getMenuItem(browseMenu);
		searchFor.setText(StaticValues.getLangString("searchFor"));
		searchFor.addSelectionListener(SelectionListener.getInstance());
		addRandomEntries = wt.getMenuItem(browseMenu);
		addRandomEntries.setText(StaticValues.getLangString("mlibAddRandom"));
		addRandomEntries.addSelectionListener(SelectionListener.getInstance());
		refreshMlib = wt.getMenuItem(browseMenu);
		refreshMlib.setText(StaticValues.getLangString("refreshMlib"));
		refreshMlib.addSelectionListener(SelectionListener.getInstance());
		browseMenu.addMenuListener(MenuListener.getInstance());
		browseTable.setMenu(browseMenu);
	}
}

