/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.log4j.Logger;
import org.x4x.core.Management;
import org.x4x.core.Playlist;
import org.x4x.core.Title;
import org.x4x.core.UpdateUIType;



/**
 * Package UpdateListener which receives update from the backend and updates the UI
 */

class PlaylistUpdateListener extends Thread implements Observer {
	private Management myManagement;
	private boolean managementUp = false;
	private boolean guiUp = false;
	private Logger logger;
	private UpdateListener parent; 
	
	/**
	 * Create and start the listener
	 * @param args 
	 *
	 */
	protected PlaylistUpdateListener(UpdateListener parent){
		logger = Logger.getLogger(getClass());
		this.parent = parent;
		myManagement = Management.getInstance(this);
		start();
	}
	
	public void run(){
		guiIsReady();
		while (true)
			try {
				sleep(20);
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
			}
	}
	
	/**
	 * called on startup to retreive default data from backend
	 *
	 */
	private void updateUI(){
		if (managementUp && guiUp){
			new Timer().schedule(new TimerTask(){
				public void run() {
					updatePlaylist();
					MainGUI.mainGui.playlistPanel.table.setSelectedIndex(myManagement.getPlaylistIndex());
				}
			},1000);
		}
	}
	
	/**
	 * Set the gui to a ready state and get startupData from Management
	 *
	 */
	protected void guiIsReady(){
		guiUp = true;
		while(!myManagement.upAndRunning()){
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
			}
		}
		managementUp = true;
		updateUI();
	}

	/**
	 * Receive updates from the Backend
	 */
	public void update(Observable arg0, Object arg1) {
		UpdateUIType uit = (UpdateUIType)arg1;
		if (managementUp && guiUp){
			switch (uit.getType()){
				case UpdateUIType.PLAYLIST_CHANGED:
					updatePlaylist();
					break;
				case UpdateUIType.BACKEND_LOST:
					updatePlaylist();
					break;
				case UpdateUIType.MLIB_ENTRY_CHANGED:
					long id = ((Long)uit.getVal()).longValue();
					List indices = myManagement.getCurrentPlaylist().indicesOfID(id);
					if (myManagement.isTitleInCache(id)){
						Title t = myManagement.getTitleForID(id);
						for (Iterator it = indices.iterator(); it.hasNext();){
							int next = ((Integer)it.next()).intValue();
							MainGUI.mainGui.playlistPanel.table.setText(new String[]{""+(next+1), 
									""+t.getID(), t.getAttribute("title"),
									t.getAttribute("artist"), t.getAttribute("album")}, next);
							myManagement.getCurrentPlaylist().updateTitleAtIndex(next, t);
							if (myManagement.getPlaylistIndex() == next)
								parent.updateForPlaybackState();
						}
					}
					break;
			}
		}
		
	}
	
	/**
	 * update playlist with current from Management 
	 *
	 */
	private void updatePlaylist() {
		Playlist pl = myManagement.getCurrentPlaylist();
		if (pl.getSize() <= 0)
			MainGUI.mainGui.playlistPanel.table.clear();
		else {
			List ids = pl.getIDs();
			List l = new ArrayList();
			for (int i = 0; i < ids.size(); i++)
				l.add(new String[]{""+(i+1), ""+ids.get(i)});
			MainGUI.mainGui.playlistPanel.table.addList(l);
			for (int i = 0; i < pl.getSize(); i++){
				Title t = pl.getTitleForIndex(i);
				if (myManagement.isTitleInCache(t.getID())){
					t = myManagement.getTitleForID(t.getID());
					pl.updateTitleAtIndex(i, t);
					MainGUI.mainGui.playlistPanel.table.setText(new String[]{""+(i+1),
							""+t.getID(), t.getAttribute("title"),
							t.getAttribute("artist"), t.getAttribute("album")}, i);
				}
			}
		}
	}
}
