/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.util.Arrays;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XTabbedPane;
import org.Xwidgets.XTable;
import org.apache.log4j.Logger;
import org.x4x.core.Management;
import org.x4x.plugins.OutputPlugin;
import org.x4x.plugins.UIPlugin;
import org.x4x.plugins.VisualPlugin;
import org.x4x.statics.StaticValues;



/**
 * PrefGUI offers a way to edit the configurations from back- and frontend
 */

class PrefGUI {
	protected XFrame myFrame;
	private WidgetFactory wt;
	
	protected XButton closeButton;
	protected XTable backendPrefs, frontendPrefs, pluginPrefs;
	protected XTabbedPane tabbed;
	private Logger logger;
	private Management myManagement;
	
	/**
	 * Create a new PrefGUI with a XTabbedPane and 2 entries, one for back- and one for frontend 
	 *
	 */
	protected PrefGUI() {
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
		wt = MainGUI.mainGui.myGUIFactory;
		myFrame = wt.getFrame(null, Styles.RESIZE);
		myFrame.setSize(700, 600);
		myFrame.disposeOnClose(false);
		myFrame.setImage(MainGUI.mainGui.iconPrefix + "trayPrefs.png");
		try {
			myFrame.setText(StaticValues.getLangString("prefGUI"));
			myFrame.setLayout(MainGUI.mainGui.layoutWithoutBorders);
		} catch (Exception e) {
			logger.debug(e.getMessage(), e);
		}
		buildMe();
	}
	
	/**
	 * update and open the window
	 *
	 */
	public void open(){
		update();
		tabbed.setSelection(0);
		myFrame.setVisible(true);
	}
	
	/**
	 * close the frame
	 *
	 */
	public void close(){
		myFrame.setVisible(false);
	}
	
	/**
	 * Construct content
	 *
	 */
	private void buildMe(){
		try {
			XLabel header = wt.getLabel(myFrame, Styles.CENTER);
			header.setText(StaticValues.getLangString("preferences").toUpperCase());
			header.setToolTipText(StaticValues.getLangString("preferencesHeader"));
			header.setLayoutData(MainGUI.mainGui.layoutDataFillHoriz);
			
			tabbed = wt.getTabbedPane(myFrame, Styles.SIMPLE);
			tabbed.setLayout(MainGUI.mainGui.layoutWithoutBorders);
			tabbed.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);			
			
			backendPrefs = wt.getTable(tabbed, Styles.BORDER);
			backendPrefs.setColumnNames(new String[]{StaticValues.getLangString("prefKey"), 
					StaticValues.getLangString("prefValue")});
			backendPrefs.setColumnWidths(new int[]{300, 300});
			backendPrefs.addMouseListener(MouseListener.getInstance());
			backendPrefs.setLayout(MainGUI.mainGui.layoutWithBorders);
			backendPrefs.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			
			frontendPrefs = wt.getTable(tabbed, Styles.BORDER);
			frontendPrefs.setColumnNames(new String[]{StaticValues.getLangString("prefKey"), 
					StaticValues.getLangString("prefValue")});
			frontendPrefs.setColumnWidths(new int[]{300, 300});
			frontendPrefs.addMouseListener(MouseListener.getInstance());
			frontendPrefs.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			frontendPrefs.setLayout(MainGUI.mainGui.layoutWithBorders);
			
			pluginPrefs = wt.getTable(tabbed, Styles.BORDER|Styles.CHECK);
			pluginPrefs.setColumnNames(new String[]{"", StaticValues.getLangString("name"), 
					StaticValues.getLangString("type"), StaticValues.getLangString("description")});
			pluginPrefs.setColumnWidths(new int[]{30, 140, 140, 450});
			pluginPrefs.addMouseListener(MouseListener.getInstance());
			pluginPrefs.addSelectionListener(SelectionListener.getInstance());
			pluginPrefs.setLayoutData(MainGUI.mainGui.layoutDataFillBoth);
			pluginPrefs.setLayout(MainGUI.mainGui.layoutWithBorders);
			
			closeButton = wt.getButton(myFrame, Styles.CENTER);
			closeButton.setText(StaticValues.getLangString("ButtonClose"));
			closeButton.addMouseListener(MouseListener.getInstance());
			closeButton.setLayoutData(MainGUI.mainGui.layoutDataFillHoriz);
			
			tabbed.addItem(frontendPrefs, StaticValues.getLangString("x4xprefs"), "");
			tabbed.addItem(pluginPrefs, "Plugins", "");
			tabbed.addItem(backendPrefs, StaticValues.getLangString("backendprefs"), "");
			
			String[] names = MainGUI.mainGui.pc.getPluginNames();
			for (int i = 0; i < names.length; i++){
				String type = "";
				Class cl = MainGUI.mainGui.pc.getPluginclass(names[i]);
				if (cl.equals(OutputPlugin.class))
					type = "OutputPlugin";
				else if (cl.equals(VisualPlugin.class))
					type = "VisualisationPlugin";
				else if (cl.equals(UIPlugin.class))
					type = "UIPlugin";
				pluginPrefs.addItem(new String[]{"", names[i], 
						type, MainGUI.mainGui.pc.getDescription(names[i])});
				if (MainGUI.mainGui.pc.isActive(names[i]))
					pluginPrefs.getItemForIndex(i).setChecked(true);
			}
			
		} catch (Throwable e){
			logger.debug(e.getMessage(), e);
		}
		update();
	}
	
	/**
	 * UpdateListener calls this method when a configval_update occurs
	 *
	 */
	protected void update() {
		String[] keys = myManagement.getConfigs();
		Arrays.sort(keys);
		int i = 0;
		for (i = 0; i < keys.length; i++){
			if (i < backendPrefs.getItemCount())
				backendPrefs.setText(new String[]{keys[i], myManagement.getConfigVal(keys[i])}, i);
			else backendPrefs.addItem(new String[]{keys[i], myManagement.getConfigVal(keys[i])});
		}
		backendPrefs.removeAllFrom(i);
		keys = StaticValues.getEditableKeys();
		Arrays.sort(keys);
		i = 0;
		for (i = 0; i < keys.length; i++){
			if (i < frontendPrefs.getItemCount())
				frontendPrefs.setText(new String[]{keys[i], StaticValues.getSetting(keys[i])}, i);
			else frontendPrefs.addItem(new String[]{keys[i], StaticValues.getSetting(keys[i])});
		}
		frontendPrefs.removeAllFrom(i);
	}
}
