/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XButton;
import org.Xwidgets.XFrame;
import org.Xwidgets.XLabel;
import org.Xwidgets.XPanel;
import org.Xwidgets.XTextField;
import org.apache.log4j.Logger;
import org.x4x.core.Title;
import org.x4x.statics.StaticValues;



/**
 * class offers a window to edit titleproperties
 */

class PropGUI {
	private XFrame myFrame;
	private WidgetFactory wt;
	
	protected XLabel labelLength, labelFileType, labelBitRate, labelSampleRate, labelID;
	protected XTextField textTitle, textArtist, textAlbum, textDate, textComment, textGenre, textURL;
	protected XButton okButton, cancelButton;
	private XPanel propPanel, butPanel, detPanel;
	private Logger logger;
	private Title current;
	
	/**
	 * Build the PropGUI
	 */
	protected PropGUI() {
		logger = Logger.getLogger(getClass());
		
		wt = MainGUI.mainGui.myGUIFactory;
		myFrame = wt.getFrame(null, Styles.SIMPLE);
		myFrame.setSize(450, 300);
		myFrame.disposeOnClose(false);
		myFrame.setImage(MainGUI.mainGui.iconPrefix + "trayIcon.png");
		buildPropPanel();
		buildButPanel();
		buildDetPanel();
		
		okButton.addMouseListener(MouseListener.getInstance());
		cancelButton.addMouseListener(MouseListener.getInstance());
		
		myFrame.setText(StaticValues.getLangString("propGUI"));
	}
	
	/**
	 * open the Window and set t as the current title
	 * @param t		Title to handle
	 */
	public void open(Title t){
		if ( t != null){
			current = t;
			myFrame.setVisible(true);
		}
	}
	
	/**
	 * Close the window and return the current Title
	 * 
	 * @return		current Title
	 */
	public Title close(){
		myFrame.setVisible(false);
		return current;
	}
	
	/**
	 * Construct editable content
	 *
	 */
	private void buildPropPanel(){
		propPanel = wt.getPanel(myFrame, Styles.BORDER);
		propPanel.setSize(450, 230);
		
		XLabel tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setLocation(30,20);
		tmp.setSize(70, 18);
		tmp.setText(StaticValues.getLangString("title") + ":");
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("artist") + ":");
		tmp.setSize(70, 18);
		tmp.setLocation(30,50);
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("album") + ":");
		tmp.setLocation(30,80);
		tmp.setSize(70, 18);
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("date") + ":");
		tmp.setLocation(30, 110);
		tmp.setSize(70, 18);
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("comment") + ":");
		tmp.setLocation(30,140);
		tmp.setSize(70, 18);
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("genre") + ":");
		tmp.setLocation(30, 170);
		tmp.setSize(70, 18);
		tmp = wt.getLabel(propPanel, Styles.LEFT);
		tmp.setText(StaticValues.getLangString("url") + ":");
		tmp.setLocation(30, 200);
		tmp.setSize(70, 18);
		
		textTitle = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textTitle.setSize(140, 22);
		textArtist = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textArtist.setSize(140, 22);
		textAlbum = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textAlbum.setSize(140, 22);
		textDate = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textDate.setSize(140, 22);
		textComment = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textComment.setSize(140, 22);
		textGenre = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textGenre.setSize(140, 22);
		textURL = wt.getTextField(propPanel, Styles.LEFT|Styles.BORDER);
		textURL.setSize(325, 22);
		
		textTitle.setLocation(105, 15);
		textArtist.setLocation(105, 45);
		textAlbum.setLocation(105, 75);
		textDate.setLocation(105, 105);
		textComment.setLocation(105, 135);
		textGenre.setLocation(105, 165);
		textURL.setLocation(105, 195);				
	}
	
	/**
	 * Construct non editable content
	 *
	 */
	private void buildDetPanel(){
		detPanel = wt.getPanel(propPanel, Styles.LEFT);
		detPanel.setSize(190, 180);
		detPanel.setLocation(260,15);
		
		(labelLength = wt.getLabel(detPanel, Styles.CENTER)).setLocation(20, 5);
		labelLength.setText(StaticValues.getLangString("length") + ": 0:00");
		labelLength.setSize(160, 22);
		(labelFileType = wt.getLabel(detPanel, Styles.CENTER)).setLocation(20, 42);
		labelFileType.setText(StaticValues.getLangString("fileType") + ": unknown");
		labelFileType.setSize(160, 22);
		(labelBitRate = wt.getLabel(detPanel, Styles.CENTER)).setLocation(20,79);
		labelBitRate.setText(StaticValues.getLangString("bitRate") + ": unknown");
		labelBitRate.setSize(160, 22);
		(labelSampleRate = wt.getLabel(detPanel, Styles.CENTER)).setLocation(20,116);
		labelSampleRate.setText(StaticValues.getLangString("sampleRate") + ": unknown");
		labelSampleRate.setSize(160, 22);
		(labelID = wt.getLabel(detPanel, Styles.CENTER)).setLocation(20,155);
		labelID.setText(StaticValues.getLangString("id") + ": unknown");
		labelID.setSize(160, 22);
	}
	
	/**
	 * Construct Buttonpanel
	 *
	 */
	private void buildButPanel(){
		butPanel = wt.getPanel(myFrame, Styles.BORDER);
		butPanel.setSize(450, 70);
		butPanel.setLocation(0,230);
		
		okButton = wt.getButton(butPanel, Styles.CENTER);
		okButton.setSize(90, 30);
		okButton.setText(StaticValues.getLangString("ButtonOk"));
		cancelButton = wt.getButton(butPanel, Styles.CENTER);
		cancelButton.setSize(90, 30);
		cancelButton.setText(StaticValues.getLangString("ButtonCancel"));
		
		okButton.setLocation(250, 2);
		cancelButton.setLocation(342, 2);
	}
}
