/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.Xwidgets.Styles;
import org.Xwidgets.WidgetFactory;
import org.Xwidgets.XComboInputBox;
import org.Xwidgets.XDirectoryChooser;
import org.Xwidgets.XFileChooser;
import org.Xwidgets.XInputBox;
import org.Xwidgets.XInputValidator;
import org.Xwidgets.XMenuItem;
import org.Xwidgets.XMessageBox;
import org.Xwidgets.XTableItem;
import org.Xwidgets.event.XSelectionListener;
import org.Xwidgets.event.XSelectionEvent;
import org.apache.log4j.Logger;
import org.x4x.core.Management;
import org.x4x.plugins.OutputPlugin;
import org.x4x.plugins.Plugin;
import org.x4x.plugins.X4XPluginException;
import org.x4x.statics.StaticValues;

/**
 * Package SelectionListener
 */
class SelectionListener extends XSelectionListener{
	private Logger logger;
	private static SelectionListener instance;
	private Management myManagement;
	private WidgetFactory wt;
	protected static String chooserDir = StaticValues.getSetting("userhome");
	
	private SelectionListener(){
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance();
		wt = MainGUI.mainGui.myGUIFactory;
	}
	
	/**
	 * Get a Listenerinstance
	 * 
	 * @return		SelectionListener instance
	 */
	public static SelectionListener getInstance(){
		if (instance == null)
			instance = new SelectionListener();
		return instance;
	}
	
	private XMenuItem isPluginMenuItem(Object o){
		for (Iterator it = MainGUI.mainGui.pluginMenuItems.iterator(); it.hasNext();){
			XMenuItem xit = (XMenuItem)it.next();
			if (xit.getRealObject().equals(o))
				return xit;
		}
		return null;
	}
	
	/**
	 * handle WidgetSelected event
	 */
	public void xWidgetSelected(XSelectionEvent e){
		int index[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
		if (MainGUI.mainGui.playlistPanel.table.indexOf(e.getSource()) != -1){
			int x = MainGUI.mainGui.playlistPanel.table.indexOf(e.getSource());
			switch (x){
				case 1:		myManagement.playlistSort("id"); break;
				case 2:		myManagement.playlistSort("title"); break;
				case 3:		myManagement.playlistSort("artist"); break;
				case 4:		myManagement.playlistSort("album"); break;
			}
		}
		else if (isPluginMenuItem(e.getSource()) != null){
			String plname = isPluginMenuItem(e.getSource()).getText().replaceFirst("\\*Plugin\\* ", "");
			Plugin tmp;
			try {
				tmp = MainGUI.mainGui.pc.getActivePlugin(plname);
				if (tmp instanceof OutputPlugin){
					List l = new ArrayList();
					int items[] = MainGUI.mainGui.playlistPanel.table.getSelectedIndices();
					org.x4x.core.Playlist current = myManagement.getCurrentPlaylist();
					for ( int i = 0; i < items.length; i++){
						l.add(current.getTitleForIndex(items[i]));
					}
					((OutputPlugin)tmp).writeList(l);
					MainGUI.mainGui.pc.runPlugin(tmp);
				}
			} catch (Exception e1) {
				logger.debug(e1.getMessage(), e1);
			}
			
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.tray)){
			System.out.println("Tray");
		}
		else if (MainGUI.mainGui.prefGui != null &&
				GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.prefGui.pluginPrefs)){
			int[] selected = MainGUI.mainGui.prefGui.pluginPrefs.getSelectedIndices();
			if (selected.length > 0){
				XTableItem xti = MainGUI.mainGui.prefGui.pluginPrefs.getItemForIndex(selected[0]);
				if (xti.isChecked()){
					try {
						if (!MainGUI.mainGui.pc.isActive(xti.getText(1)))
							GlobalEvents.activatePlugin(xti.getText(1));
					} catch (X4XPluginException e1) {
						xti.setChecked(false);
						logger.debug(e1.getMessage(), e1);
					}
				}
				else {
					try {
						if (MainGUI.mainGui.pc.isRunning(xti.getText(1))){
							Plugin pl = MainGUI.mainGui.pc.getActivePlugin(xti.getText(1));
							if (pl != null){
								MainGUI.mainGui.pc.stopPlugin(pl);
							}
						}
						if (MainGUI.mainGui.pc.isActive(xti.getText(1)))
							GlobalEvents.deactivatePlugin(xti.getText(1));
					} catch (X4XPluginException e1) {
						xti.setChecked(false);
						logger.debug(e1.getMessage(), e1);
					}
				}
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.delItemFromPl)){
			GlobalEvents.deletePlaylistItems(index);
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.searchBox)){
			String entry = GlobalEvents.getTextFromComboIndex(MainGUI.mainGui.mlibPanel.searchBox);
			MainGUI.mainGui.mlibPanel.preSearchbox.setToolTipText(StaticValues.getLangString("searchFor"));
			MainGUI.mainGui.mlibPanel.textSearch.setToolTipText(StaticValues.getLangString("searchFor"));
			MainGUI.mainGui.mlibPanel.textSearch.setText("");
			MainGUI.mainGui.mlibPanel.strict.setEnabled(true);
			if (entry.equals("all")){
				MainGUI.mainGui.mlibPanel.textSearch.setText("");
				MainGUI.mainGui.mlibPanel.textSearch.setEnabled(false);
				MainGUI.mainGui.mlibPanel.searchButton.setEnabled(false);
				MainGUI.mainGui.mlibPanel.strict.setEnabled(false);
				myManagement.mlibSelect("", true);
			}
			else {
				MainGUI.mainGui.mlibPanel.textSearch.setEnabled(true);
				MainGUI.mainGui.mlibPanel.searchButton.setEnabled(true);
				MainGUI.mainGui.mlibPanel.strict.setEnabled(true);
				MainGUI.mainGui.mlibPanel.searchTable.clear();
			}
			/*if (entry.equals("ownQuery")){
				MainGUI.mainGui.mlibPanel.preSearchbox.setToolTipText(myManagement.getDefaultSQLString() + " ..");
				MainGUI.mainGui.mlibPanel.textSearch.setToolTipText(myManagement.getDefaultSQLString() + " ..");
				MainGUI.mainGui.mlibPanel.strict.setEnabled(false);
			}*/
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.clearPlaylist)){
			XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK|Styles.CANCEL|Styles.QUESTION);
			box.setText(StaticValues.getLangString("reallyClearPlaylist"));
			box.open();
			if (box.getButton() == Styles.OK)
				myManagement.playlistClear();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.showTitleProperties)){
			if (index.length > 0)
				GlobalEvents.showTitleInfos(myManagement.getTitleForID(Long.parseLong(
					MainGUI.mainGui.playlistPanel.table.getItemForIndex(index[0]).getText(1))));
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.titleInfo)){
			int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
			if (items.length > 0)
				GlobalEvents.showTitleInfos(myManagement.getTitleForID(Long.parseLong(
					MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex(items[0]).getText(0))));
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.browseNetwork)){
			MainGUI.mainGui.CIFSBrowser.open(MainGUI.mainGui.mlibPanel.browseNetwork,
					StaticValues.getLangString("mediafiles"), myManagement.getFileTypes());
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.browseNetwork)){
			MainGUI.mainGui.CIFSBrowser.open(MainGUI.mainGui.playlistPanel.browseNetwork, 
					StaticValues.getLangString("mediafiles"), myManagement.getFileTypes());
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.savePlaylist)){
			GlobalEvents.savePlaylist();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.shufflePlaylist)) 
			myManagement.playlistShuffle();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.menuExit) 
				|| GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayExit))
			MainGUI.mainGui.finalizer();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.menuPrefs))
			MainGUI.mainGui.prefGui.open();
		// TODO make that better asap
		/*else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.movieEntry)){
			XFileChooser xfc = wt.getFileChooser(MainGUI.mainGui.
					myFrame, Styles.FD_OPEN|Styles.SINGLE);
			xfc.setSelectedPath(chooserDir);
			xfc.addFileFilter("Movies", new String[]{"avi", "asf"});
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			String[] selected = xfc.getSelectedFiles();
			if (selected.length > 0 && !selected[0].equals("")){
				try {
					new X4XMoviePlayer(new File(xfc.getSelectedPath() + File.separator + selected[0]).toURL());
				} catch (MalformedURLException e1) {
					logger.debug(e1.getMessage(), e1);
				}
			}
		}*/
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.menuMlib)){
			if (MainGUI.mainGui.tabbedPane != null)
				MainGUI.mainGui.tabbedPane.setSelection(0);
			MainGUI.mainGui.lists.setVisible(true);
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.aboutEntry) 
				|| GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayAbout))
			new AboutWindow();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayPlay))
			myManagement.play();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayStop))
			myManagement.stop();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayPause))
			myManagement.pause();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayNext))
			myManagement.next();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.trayPrev))
			myManagement.prev();
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.helpEntry)){
			XMessageBox box = wt.getMessageBox(MainGUI.mainGui.myFrame, Styles.OK);
			box.setText(StaticValues.getLangString("noHelpAtm"));
			box.open();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.eqEntry)){
			MainGUI.mainGui.buildEQ();
			MainGUI.mainGui.eqWindow.setVisible(true);
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.browseBox)){
			GlobalEvents.browseMlib();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.loadPlaylistFromFile)){
			XFileChooser xfc = wt.getFileChooser(MainGUI.mainGui.
					myFrame, Styles.FD_OPEN|Styles.SINGLE);
			xfc.setSelectedPath(chooserDir);
			xfc.addFileFilter(StaticValues.getLangString("playlists"), myManagement.getPlaylistTypes());
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			String[] selected = xfc.getSelectedFiles();
			if (selected.length > 0 && !selected[0].equals("")){
				String complete = xfc.getSelectedPath() + File.separator + selected[0];
				myManagement.mlibLoadPlaylist("file://" + complete, 
						selected[0].substring(0,selected[0].indexOf(".")));
			}
			myManagement.mlibGetPlaylists();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.searchFor)){
			GlobalEvents.handleMlibBrowsing(MainGUI.mainGui.mlibPanel.browseTable.getSelectedIndices());
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.addEntryMedialib)){
			String[] selected = readFilesViaChooser();
			if (selected.length > 0){
				List l = new ArrayList();
				for (int i = 0; i < selected.length; i++){
					if (!isPlaylistType(selected[i]))
						l.add("file://" + selected[i]);
					else {
						myManagement.mlibLoadPlaylist("file://" + selected[i], 
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
					}
				}
				myManagement.mlibAddURLs(l);
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.addRandomEntries)){
			XComboInputBox box = wt.getComboInputBox(MainGUI.mainGui.myFrame, Styles.READ_ONLY);
			box.setText(StaticValues.getLangString("mlibAddRandom"));
			box.setName(StaticValues.getLangString("mlibAddRandom"));
			String array[] = new String[300];
			for ( int i = 0; i < array.length; i++)
				array[i] = ""+(i+1);
			box.setData(array);
			box.setIndex(19);
			box.open();
			if (box.getButton() == Styles.OK && box.getIndex() > 0){
				String query = "WHERE key='url' ORDER BY RANDOM() LIMIT " + box.getIndex();
				myManagement.mlibAddQueryToPlaylist(query);
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.
							tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.addDirectory)){
			XDirectoryChooser xfc = wt.getDirectoryChooser(MainGUI.mainGui.myFrame);
			xfc.setSelectedPath(chooserDir);
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			logger.debug("chosen path: " + xfc.getSelectedPath());
			if (!xfc.getSelectedPath().equals("")){
				myManagement.mlibAddPath(xfc.getSelectedPath());
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.addDirToPl)){
			XDirectoryChooser xfc = wt.getDirectoryChooser(MainGUI.mainGui.myFrame);
			xfc.setSelectedPath(chooserDir);
			xfc.open();
			chooserDir = xfc.getSelectedPath();
			logger.debug("chosen path: " + xfc.getSelectedPath());
			if (!xfc.getSelectedPath().equals("")){
				myManagement.addPathToPlaylist(xfc.getSelectedPath());
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.refreshMlib)){
			myManagement.mlibRefresh();
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.addFileToPl)){
			String[] selected = readFilesViaChooser();
			if (selected.length > 0)
				for (int i = 0; i < selected.length; i++){
					if (!isPlaylistType(selected[i]))
						myManagement.addTitleToPlaylist("file://" + selected[i]);
					else {
						myManagement.mlibLoadPlaylist("file://" + selected[i], 
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
						myManagement.mlibAddPlaylistToCurrent(
								selected[i].substring(selected[i].lastIndexOf("/")+1,selected[i].indexOf(".")));
					}
				}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.addUrlToPl)){
			XInputBox in = wt.getInputBox(null, Styles.QUESTION);
			in.setName(StaticValues.getLangString("addURL"));
			in.setText(StaticValues.getLangString("addURL"));
			in.open();
			if (in.getButton() == Styles.OK){
				String text = in.getInput();
				myManagement.addTitleToPlaylist(text);
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.mlibPanel.addToPlaylist)) {
			int items[] = MainGUI.mainGui.mlibPanel.searchTable.getSelectedIndices();
			if (items.length > 0){
				List l = new ArrayList();
				for ( int i = 0; i < items.length; i++)
					l.add(new Long(MainGUI.mainGui.mlibPanel.searchTable.getItemForIndex(items[i]).getText(0)));
				myManagement.addIDsToPlaylist(l);
				if (MainGUI.mainGui.tabbedPane != null)
					MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
			}
		}
		else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.newPlaylist)) {
			XInputBox newName = wt.getInputBox(null, Styles.QUESTION);
			newName.setInput(StaticValues.getLangString("newPlaylistName"));
			newName.setName((StaticValues.getLangString("enterNewPlaylistName")));
			newName.setText((StaticValues.getLangString("enterNewPlaylistName")));
			newName.setInputValidator(new XInputValidator(){
				public String isValid(String text) {
					if (text.startsWith("_")) return StaticValues.getLangString("noUnderscores");
					if (text.length() < 3) return StaticValues.getLangString("tooShort");
					return null;
				}
			});
			newName.open();
			
			if (newName.getButton() == Styles.OK){
				String name = newName.getInput();
				myManagement.mlibNewPlaylist(name);
				myManagement.mlibLoadPlaylist(name);
				myManagement.mlibGetPlaylists();
			}
		} // end new
		
		// playlist
		else if (MainGUI.mainGui.playlistPanel.playlistTable.getItemCount() > 0) {
			int items[] = MainGUI.mainGui.playlistPanel.playlistTable.getSelectedIndices();
			if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.openPlaylist)) {
				if (items.length > 0) {
					myManagement.mlibLoadPlaylist(MainGUI.mainGui.playlistPanel.playlistTable.
							getItemForIndex(items[0]).getText(1));
					if (MainGUI.mainGui.tabbedPane != null)
						MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
				}
			}
			
			else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.addPlaylist)) {
				if (items.length > 0){
					myManagement.mlibAddPlaylistToCurrent(MainGUI.mainGui.playlistPanel.playlistTable.
						getItemForIndex(items[0]).getText(1));
					if (MainGUI.mainGui.tabbedPane != null)
						MainGUI.mainGui.tabbedPane.setSelection(MainGUI.mainGui.
								tabbedPane.getIndex(MainGUI.mainGui.playlistPanel.panel));
				}
			}
			
			else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.delPlaylist)) {
				if ( items.length > 0){
					GlobalEvents.deletePlaylists(items);
				}
			}
			
			else if (GlobalEvents.checkSource(e.getSource(), MainGUI.mainGui.playlistPanel.renamePlaylist)) {
				GlobalEvents.renamePlaylist(items);
			} // end rename
		}
	}
	
	/**
	 * check if the given string ends with a playlist-type
	 * 
	 * @param name
	 * @return		true if playlist, false otherwise
	 */
	private boolean isPlaylistType(String name){
		String playlists[] = myManagement.getPlaylistTypes();
		for (int i = 0; i < playlists.length; i++)
			if (name.endsWith(playlists[i]))return true;
		return false;
	}

	/**
	 * Read files via a XFileChooser
	 * 
	 * @return		read elements as String with path and name
	 */
	private String[] readFilesViaChooser() {
		XFileChooser xfc = wt.getFileChooser(MainGUI.mainGui.
				myFrame, Styles.FD_OPEN|Styles.MULTI);
		xfc.setSelectedPath(chooserDir);
		xfc.addFileFilter(StaticValues.getLangString("mediafiles"), myManagement.getAllSupportedTypes());
		xfc.open();
		String[] selected = xfc.getSelectedFiles();
		chooserDir = xfc.getSelectedPath();
		for (int i = 0; i < selected.length; i++)
			selected[i] = xfc.getSelectedPath() + File.separator + selected[i];
		return selected;
	}
	
	/**
	 * Handle DefaultWidgetSelected event
	 */
	public void xWidgetDefaultSelected(XSelectionEvent arg0) {}
}
