/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Copyright:
 * Georg Schild (dangertools@gmail.com)
 * Franz Endstrasser (franz.endstrasser@gmail.com)
 ******************************************************************/

package org.x4x.gui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Timer;
import java.util.TimerTask;

import org.Xwidgets.XSlider;
import org.apache.log4j.Logger;
import org.x4x.core.KeepStateThread;
import org.x4x.core.Management;
import org.x4x.core.PlaylistInformation;
import org.x4x.core.Title;
import org.x4x.core.UpdateUIType;
import org.x4x.statics.StaticValues;

/**
 * Package UpdateListener which receives update from the backend and updates the UI
 */

class UpdateListener extends Thread implements Observer {
	private Management myManagement;
	private boolean managementUp = false;
	private boolean guiUp = false;
	private Logger logger;
	private char roller[] = new char[]{45, 92, 124, 47};
	private int rollerPosition = 0;
	private Timer roll;
	private TimerTask rollerTask;
	private Title current;
	private String[] args;
	
	/**
	 * Create and start the listener
	 * @param args 
	 *
	 */
	protected UpdateListener(String args[]){
		logger = Logger.getLogger(getClass());
		myManagement = Management.getInstance(this);
		new PlaylistUpdateListener(this);
		this.args = args;
		start();
	}
	
	public void run(){
		handleArgs();
		guiIsReady();
		while (true)
			try {
				sleep(20);
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
			}
	}
	
	/**
	 * called on startup to retreive default data from backend
	 *
	 */
	private void updateUI(){
		if (managementUp && guiUp){
			new Timer().schedule(new TimerTask(){
				public void run() {
					myManagement.updateAll();
					updateForPlaybackState();
					updateSliders(-1);
					myManagement.mlibGetPlaylists();
					updateRepeatEtc();
				}
			},1000);
			new Timer().schedule(new TimerTask(){
				public void run() {
					GlobalEvents.browseMlib();
				}
			}, 4000);
		}
	}
	/**
	 * Called for handling arguments on startup
	 * 
	 */
	private void handleArgs(){
		if (args != null && args.length > 0){
			int startAt = 0;
			if (args[0].equals("--clean")){
				myManagement.playlistClear();
				startAt++;
			}
			String supported[] = myManagement.getFileTypes();
			String playlists[] = myManagement.getPlaylistTypes();
			for (int i = startAt; i < args.length; i++){
				for ( int j = 0; j < supported.length; j++)
					if (args[i].toLowerCase().endsWith(supported[j]))
						myManagement.addTitleToPlaylist("file://" + args[i]);
				for ( int j = 0; j < playlists.length; j++)
					if (args[i].toLowerCase().endsWith(playlists[j])){
						myManagement.mlibLoadPlaylist("file://" + args[i], 
								args[i].substring(args[i].lastIndexOf("/")+1,args[i].indexOf(".")));
						myManagement.mlibAddPlaylistToCurrent(
								args[i].substring(args[i].lastIndexOf("/")+1,args[i].indexOf(".")));
					}
			}
		}
	}
	
	/**
	 * Set the gui to a ready state and get startupData from Management
	 *
	 */
	protected void guiIsReady(){
		guiUp = true;
		while(!myManagement.upAndRunning()){
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				logger.debug(e.getMessage(), e);
			}
		}
		roll = new Timer();
		managementUp = true;
		updateUI();
	}

	/**
	 * Receive updates from the Backend
	 */
	public void update(Observable arg0, Object arg1) {
		UpdateUIType uit = (UpdateUIType)arg1;
		if (managementUp && guiUp){
			switch (uit.getType()){
				case UpdateUIType.PLAYBACK_ID_CHANGED:
					updateForPlaybackState();
					break;
				case UpdateUIType.PLAYLIST_CURRENT_POSITION_CHANGED:
					MainGUI.mainGui.playlistPanel.table.setSelectedIndex(myManagement.getPlaylistIndex());
					updateForPlaybackState();
					break;
				case UpdateUIType.PLAYBACK_STATUS_CHANGED:
					updateForPlaybackState();
					break;
				case UpdateUIType.CONFIGVAL_CHANGED:
					MainGUI.mainGui.prefGui.update();
					break;
				case UpdateUIType.PLAYBACK_SETTING_CHANGED:
					updateRepeatEtc();
					break;
				case UpdateUIType.PLAYTIME_CHANGED:
					int min, sec;
					min = ((Double)uit.getVal()).intValue()/60;
					sec = ((Double)uit.getVal()).intValue()%60;
					if (MainGUI.mainGui.playtime != null){
						MainGUI.mainGui.playtime.setPosition(min*60+sec);
						MainGUI.mainGui.playtime.setToolTipText(min + ":" + ((sec<10)?"0":"") + sec);
					}
					if (current != null){
						int min2 = current.getLength()/60;
						int sec2 = current.getLength()%60;
						MainGUI.mainGui.tray.setTooltipText(current.getAttribute("artist") + "\n" + current.getAttribute("title") + "\n" +
								min + ":" + ((sec<10)?"0":"") + sec + " " + 
								StaticValues.getLangString("of") + " " + min2 + ":" + ((sec2<10)?"0":"") + sec2);
						MainGUI.mainGui.currentProgress.setText(min + ":" + ((sec<10)?"0":"") + sec + " " + 
								StaticValues.getLangString("of") + " " + min2 + ":" + ((sec2<10)?"0":"") + sec2);
					}
					else current = myManagement.getPlayingTitle();
					break;
				case UpdateUIType.VOLUME_OR_EQUALIZER_CHANGED:
					if (uit.getVal() instanceof String){
						if (uit.getVal().equals("p"))
							if (MainGUI.mainGui.eqPanel != null){
								MainGUI.mainGui.eqPanel.preamp.setPosition((myManagement.getPreamp()-50)*(-1));
								MainGUI.mainGui.eqPanel.preamp.setToolTipText("Preamp: "+myManagement.getPreamp());
							}
					}
					else
						updateSliders(((Integer)uit.getVal()).intValue());
					break;
				case UpdateUIType.BACKEND_LOST:
					MainGUI.mainGui.playtime.setMax(0);
					MainGUI.mainGui.playtime.setPosition(0);
					MainGUI.mainGui.playtime.setToolTipText("0:00");
					MainGUI.mainGui.currentProgress.setText("");
					MainGUI.mainGui.tray.setTooltipText("X4X");
					updateFlow(StaticValues.getLangString("backendDisconnected"));
					break;
				case UpdateUIType.MLIB_LIST:
					List entries = (List)uit.getVal();
					Collections.sort(entries);
					MainGUI.mainGui.mlibPanel.browseTable.setSelectedIndex(-1);
					MainGUI.mainGui.mlibPanel.browseTable.addList(entries);
					break;
				case UpdateUIType.MLIB_SEARCH:
					entries = (List)uit.getVal();
					ArrayList list = new ArrayList();
					
					Collections.sort(entries);
					for (int i = 0; i < entries.size(); i++){
						Title t = (Title)entries.get(i);
						list.add(new String[]{""+t.getID(), t.getAttribute("artist"),
							t.getAttribute("title"), t.getAttribute("album"), t.getAttribute("genre")});
					}
					MainGUI.mainGui.mlibPanel.searchTable.setSelectedIndex(-1);
					MainGUI.mainGui.mlibPanel.searchTable.addList(list);
					if (MainGUI.mainGui.tabbedPane != null)
						MainGUI.mainGui.tabbedPane.setSelection(MainGUI.
								mainGui.tabbedPane.getIndex(MainGUI.mainGui.mlibPanel.searchPanel));
					break;
				case UpdateUIType.PLAYLIST_LOADED:
					myManagement.mlibGetPlaylists();
					break;
				case UpdateUIType.PLAYLIST_INFO:
					List pentries = (List)uit.getVal();					
					MainGUI.mainGui.playlistPanel.playlistTable.clear();
					
					for (ListIterator it = pentries.listIterator(); it.hasNext();){
						PlaylistInformation pi = (PlaylistInformation)it.next();
						MainGUI.mainGui.playlistPanel.playlistTable.addItem(new String[]{"" +
								pi.getId(), pi.getName(), "" + pi.getItems()});
					}
					break;
				case UpdateUIType.BG_WORKING:
					updateForPlaybackState();
					break;
			}
		}
		
	}
	
	/**
	 * Update flowtext/status after getting current playbackstate
	 *
	 */
	protected void updateForPlaybackState() {
		if (myManagement.getBackgroundWorkerStatus() != UpdateUIType.BG_WORKING){
			if (rollerTask != null){
				rollerTask.cancel();
				rollerTask = null;
			}
			rollerPosition = 0;
			switch (myManagement.getPlaybackStatus()){
				case KeepStateThread.PAUSED:
					updateFlow(StaticValues.getLangString("paused"));
					break;
				case KeepStateThread.PLAYING:
					updateFlow("");
					break;
				default:
					updateFlow(StaticValues.getLangString("stopped"));
				MainGUI.mainGui.playtime.setPosition(0);
				MainGUI.mainGui.playtime.setToolTipText("0:00");
				MainGUI.mainGui.currentProgress.setText("");
				break;
			}
		}
		else {
			if (rollerTask == null){
				rollerTask = new TimerTask(){
					public void run() {
						if (rollerPosition > 3)
							rollerPosition=0;
						MainGUI.mainGui.status.setText(StaticValues.getLangString("backgroundWorking").toUpperCase()
								+ "  " + roller[rollerPosition++]);
					}
				};
				roll.schedule(rollerTask, 0, 50);
			}
		}
	}
	
	/**
	 * update sliders volume or equalizer
	 * 
	 * @param x		if -1 we'll update volume, else equalizer x
	 */
	private void updateSliders(int x){
		if ( x == -1 && MainGUI.mainGui.volume != null){
			MainGUI.mainGui.volume.setPosition(myManagement.getVolume());
			MainGUI.mainGui.volume.setToolTipText(StaticValues.getLangString("volume") + ": "+myManagement.getVolume());
			if (MainGUI.mainGui.currentVolume != null)
				MainGUI.mainGui.currentVolume.setText(StaticValues.getLangString("volume") + ": "+myManagement.getVolume());
		}
		else {
			if (MainGUI.mainGui.eqPanel != null){
				try {
					((XSlider)MainGUI.mainGui.eqPanel.sliders.get(x)).setPosition((myManagement.getEQ(x)-50)*(-1));
					((XSlider)MainGUI.mainGui.eqPanel.sliders.get(x)).setToolTipText(""+myManagement.getEQ(x));
				} catch (Exception e){
					logger.debug(e.getMessage(), e);
				}
			}
		}
	}
	
	/**
	 * update flowtext with given text
	 * 
	 * @param text		if empty we'll get artist and title for current playing title, else we set text
	 */
	private void updateFlow(String text){
		if (text.equals("")){
			current = myManagement.getPlayingTitle();
			if (MainGUI.mainGui.flow != null)
				MainGUI.mainGui.flow.setFlowText(current.getAttribute("artist") + " - " + current.getAttribute("title"));
			if (MainGUI.mainGui.playtime != null)
				MainGUI.mainGui.playtime.setMax(current.getLength());
			int min = current.getLength()/60;
			int sec = current.getLength()%60;
			MainGUI.mainGui.tray.setTooltipText(current.getAttribute("artist") + "\n" + current.getAttribute("title") + "\n" +
					min + ":" + ((sec<10)?"0":"") + sec);
			MainGUI.mainGui.status.setText(StaticValues.getLangString("playing") + ": " + current.getAttribute("artist") 
					+ " - " + current.getAttribute("title"));
		}
		else{
			if (MainGUI.mainGui.flow != null)
				MainGUI.mainGui.flow.setFlowText("");
			MainGUI.mainGui.status.setText(text);
			MainGUI.mainGui.tray.setTooltipText("X4X - " + text);
		}
	}
	
	private void updateRepeatEtc(){
			MainGUI.mainGui.shuffle.setImage(MainGUI.mainGui.iconPrefix + "random" + 
					(myManagement.isRandomizing()?"Active":"") + ".png");
			MainGUI.mainGui.repeatOne.setImage(MainGUI.mainGui.iconPrefix + "repeatOne" + 
					(myManagement.isRepeatOneEnabled()?"Active":"") + ".png");
			MainGUI.mainGui.repeatAll.setImage(MainGUI.mainGui.iconPrefix + "repeatAll" + 
					(myManagement.isRepeatAllEnabled()?"Active":"") + ".png");
			if (MainGUI.mainGui.eqPanel != null)
				MainGUI.mainGui.eqPanel.enabled.setChecked(myManagement.isMixerEnabled());
		
	}
}
