/******************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************/

package org.x4x.plugins;

import java.util.Observable;
import java.util.Properties;
import java.util.Timer;
import java.util.TimerTask;

import org.Xwidgets.WidgetFactory;
import org.apache.log4j.Logger;

public abstract class Plugin extends Observable implements Runnable{
	protected boolean inited = false;
	protected boolean running = false;
	protected int ttl = 0;
	protected Logger logger;
	protected WidgetFactory wf;
	private Thread t;
	protected Properties config;
	
	public Plugin(){
		logger = Logger.getLogger(getClass());
	}
	
	protected abstract void init(Properties configuration) throws X4XPluginException;
	protected abstract void destroy() throws X4XPluginException;
	protected abstract void start() throws X4XPluginException;
	protected abstract void stop() throws X4XPluginException;
	public abstract String getName();
	public abstract String getDescription();
	
	public Properties getSettings(){
		return config;
	}
	
	protected final void setThread(Thread t){
		this.t = t;
	}
	
	public final void run(){
		setChanged();
		if (!inited) {
			notifyObservers("not inited");
			return;
		}
		else if (running) {
			notifyObservers("already running");
			return;
		}
		else {
			try {
				Timer clock = null;
				if (ttl > 0){
					clock = new Timer();
					clock.schedule(new TimerTask(){
						public void run() {
							try{
								if (t.isAlive() || running){
									stop();
									t.interrupt();
									notifyObservers("canceled");
								}
							} catch (Exception e){
								logger.debug(e.getMessage(), e);
							}
						}
					}, ttl);
				}
				start();
				if (clock != null) clock.cancel();
				notifyObservers("");
			} catch (X4XPluginException e) {
				logger.debug(e.getMessage(), e);
				notifyObservers(e.getMessage());
			}
		}
	}
}
